#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>

#include <cdefs.h>
#include <rccl.h>
#include <grf4.h>
#include <timeval.h>

#include "accTrack.h"

#define MARGIN	 10.0
#define X_WIDTH	100.0
#define Y_WIDTH	100.0

double centerX;
double centerY;
double scaleX;
double scaleY;

double targetX = 0;
double targetY = 0;
double posX = 0;
double posY = 0;

int winw;
int winh;
int winx;
int winy;

Grf4Object *disc;

int clickMode = 0;
int filterMode = 0;
int robotConnect = 0;
int targetLocked = 0;
int graphicsInterval = 50; /* msec */

MotionContext ctx;
MotionState state;

static void resize()
{
	centerX = winx + winw/2;
	centerY = winy + winh/2;
	scaleX = (X_WIDTH+2*MARGIN)/(double)MIN(winw,winh);
	scaleY = (Y_WIDTH+2*MARGIN)/(double)MIN(winw,winh);
}

void getCursor(xt, yt)
double *xt, *yt;
{
	Grf4Event evx;

	grf4PollEvent (GRF_CURSOR, &evx);
	*xt = (evx.val1 - centerX)*scaleX;
	*yt = (evx.val2 - centerY)*scaleY;
}

static handleEvent(ev)
Grf4Event *ev;
{
	switch(ev->code)
	 { 
	   default:
	    { break;
	    }
	   case GRF_RESIZE:
	    { winw = ev->val1;
	      winh = ev->val2;
printf ("%d %d\n", winw, winh);
	      resize();
	      break;
	    }
	   case GRF_REDRAW:
	    { break;
	    }
	   case GRF_RIGHT_MOUSE:
	    { if (clickMode)
	       { getCursor (&targetX, &targetY);
	       }
	      break;
	    }
	   case GRF_LEFT_MOUSE:
	    { if (ev->val1)
	       { /* button pressed */
		 targetLocked = -1;
	       }
	      else
	       { /* button released */
		 targetLocked = 0;
	       }
	      break;
	    }
	 }
}

onQuit(n,code)
int n;
int code;
{
	if (code == BF_PRESSED)
	 { 
	   if (robotConnect == 2)
	    { rcclRelease (1);
	    }
	   exit (0);
	 }
}

void drawIt()
{
	double x, y;
	Grf4Event ev;
	float a;

	grf4Clear ();

	grf4Move2 (-X_WIDTH/2, -Y_WIDTH/2);
	grf4Draw2 ( X_WIDTH/2, -Y_WIDTH/2);
	grf4Draw2 ( X_WIDTH/2,  Y_WIDTH/2);
	grf4Draw2 (-X_WIDTH/2,  Y_WIDTH/2);
	grf4Draw2 (-X_WIDTH/2, -Y_WIDTH/2);

	grf4PushTrans();
	grf4Translate (posX, posY, 0.0);
	grf4DrawObject (disc);
	grf4PopTrans();

	grf4Flush ();
}

computeCycle()
{
	double x, y;

	if (!clickMode)
	 { getCursor (&x, &y);
	   if (targetLocked == 0)
	    { if ((x-targetX)*(x-targetX) + (y-targetY)*(y-targetY) < 2)
	       { targetLocked = 1;
	       }
	    }
	   if (targetLocked > 0)
	    { targetX = x;
	      targetY = y;
	    }
	   if (!robotConnect)
	    { posX = targetX;
	      posY = targetY;
	    }
	 }
	drawIt();
}

char **argvec;

doAlter(tr, arg, mnp)
TRSF *tr;
int arg;
MANIP *mnp;
{
	if (filterMode)
	 { VECT pos;

	   pos.x = targetX;
	   pos.y = targetY;
	   pos.z = 0;
	   accTrack (&state, &pos, &state, &ctx);
	   posX = state.pos.x;
	   posY = state.pos.y;
	 }
	else
	 { posX = targetX;
	   posY = targetY;
	 }
	tr->p.x = -posY;
	tr->p.y = -posX;
}

main(argc, argv)
int argc;
char **argv;
{
	struct timeval nextTime;
	POS *pos;
	TRSF *b, *alter;
	MANIP *mnp;

	long flags;
	int i;

	flags = GRF_2DONLY | GRF_X11;

	argvec = argv;
	argc--; argv++;
	while (argc > 0)
	 { if (ARGUMENT ("-click"))
	    { clickMode = 1;
	    }
	   else if (ARGUMENT ("-filter"))
	    { filterMode = 1;
	    }
	   else if (ARGUMENT ("-rbt"))
	    { robotConnect = 1;
	    }
	   else if (ARGUMENT ("-interval"))
	    { argc--; argv++;
	      if (argc <= 0)
	       { goto usage;
	       }
	      graphicsInterval = atoi (*argv);
	    }
	   else
	    { goto usage;
	    }
	   argc--; argv++;
	   continue;
usage:
	   fprintf (stderr, 
"Usage: %s [-click] [-filter] [-rbt] [-interval]\n", 
		    argvec[0]);
	   exit (1);
	 }

	grf4AddButton ("quit", 0, 1, onQuit, 0);

	if (grf4Init (argvec[0], GRF_WHITE, GRF_BLACK, flags) < 0)
	 { printf ("Can't init graphics\n");
	   exit (-1);
	 }
	grf4SetDrawFxn (drawIt, 0);

	grf4Color (0xffffff);
	grf4GetWindowSize (&winw, &winh);

	grf4QueueEvent (GRF_BUTTON_EVENT);
	grf4QueueEvent (GRF_REDRAW_EVENT);
	grf4QueueEvent (GRF_RESIZE_EVENT);

	grf4Ortho (-MARGIN-X_WIDTH/2, X_WIDTH/2+MARGIN, 
		   -MARGIN-Y_WIDTH/2, Y_WIDTH/2+MARGIN, 
		    -1.0, 1.0, 1.0);

	resize();

#define NSIDES	10

	disc = grf4GenRegPolygon (NSIDES, /*sidelen=*/2.0, (void*)0);

	drawIt();

	if (robotConnect)
	 { 
	   rcclSetOptions (RCCL_ERROR_EXIT);
	   b = allocTrans (NULL, UNDEF);
	   alter = allocTrans (NULL, UNDEF);
	   pos = makePosition ("Pos", T6, EQ, b, alter);

	   mnp = rcclCreate (getDefaultRobot(), 0);
	   rcclStart ();
	   rcclBlock (); /* paranoid */

	   robotConnect++;

	   ctx.velMax = 100;
	   ctx.accMax = 1000;
	   ctx.rate = 1000.0/rcclGetInterval();
	   zeroVect (&state.pos);
	   zeroVect (&state.vel);

	   readTrans (mnp->t6, b);
	   setMod (mnp, 'c');
	   setTime (mnp, F_DEFAULT, F_UNDEF);
	   transMotionEval (alter, doAlter, 0, mnp);
	   move (mnp, pos);
	 }

	timerclear (&nextTime);
	while (1)
	 { grf4CycleWait (graphicsInterval, &nextTime, handleEvent);
	   computeCycle ();
	 }
	if (robotConnect)
	 { rcclRelease (1);
	 }
}
