/*-=-=-=-=-=-=-=-=-=-=-=- RCCL/RCI Notice of Copyright -=-=-=-=-=-=-=-=-=-=-*
 *									    *
 *	Copyright (C) 1987, 1991 by John E. Lloyd.			    *
 *									    *
 *	Information on copyright permissions, as well as a complete	    *
 *	disclaimer of warranty, is given in the file COPYRIGHT.		    *
 *									    *
 *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

#if defined(Sun4sol_TARGET) && defined(UBC_CS)

#include "../h/cdefs.h"
#include "../h/IOdevice.h"
#include "../h/errorCodes.h"
#include "../h/errorStack.h"
#include "../h/s16dTspDevice.h"
#include "../h/packByte.h"

#include "../drivers/driErrorMsgs.h"

#define DEFAULT_TIMEOUT		10 /* seconds */
#define FIXED_REPLY_SIZE	0
#define READ_CHECKSUM		1 /* compute checksum on reads */
#define WRITE_CHECKSUM		1 /* compute checksum on writes */

/* Note: Checksum computation asymmetrical. When we *read*, we
   expect a packet like this:

	size (1 byte) + (<size> bytes) + checksum (2 bytes)

   and the checksum is computed on the <size> bytes.

   When we write, the outgoing packet looks like

	(<size> bytes) + checksum (2 bytes)

   and the checksum is computed on the <size> bytes. */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include "s16dreg.h"
#include "s16dioctl.h"
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/file.h>
#include <unistd.h>

#include <errno.h>

#define LINKNAME	"/dev/RCIs16dmem0"

static S16dRegs *regsPtr;

int S16DOpenLink( char *name )
{
	char *fxnName = "S16DOpenLink()";
	int link;
	unsigned short ushort;
	unsigned int uint;
	volatile int delayCnt;

	/* Stuff for mapping the beast into memory */
	unsigned long regAddr = 0;
	int regSize = 20;
	int pageSize;
	off_t pageStart;
	caddr_t vpageStart;
	u_long pageOffset;

	if (!name)
	 { name = LINKNAME;
	 }
	link = open (name, O_RDWR);
	if (link < 0 )
	 { sysErrorMsg (EConnect, fxnName, "open");
	   return (-1);
	 }

	/* Now map the thing into memory */

#ifdef __STDC__
    	pageSize = sysconf(_SC_PAGESIZE);
#else
    	pageSize = getpagesize();
#endif
	pageStart = (regAddr / pageSize) * pageSize;
	pageOffset = regAddr % pageSize;
	vpageStart = mmap((caddr_t)0, regSize, PROT_READ|PROT_WRITE,
			   MAP_SHARED, link, pageStart);
	if (vpageStart == (caddr_t)(-1))
	 { sysErrorMsg (EConnect, fxnName, "mmap");
	   return (-1);
	 }

	regsPtr = (S16dRegs *)(vpageStart + pageOffset);

#if 1
	/* pulse FUNCT1 to make sure the interface logic is reset.
	   Also, issue a reset command to the board. */
	regsPtr->csr = S16D_RESET | S16D_FNCT1;
	for (delayCnt=0; delayCnt<100; delayCnt++)
	   ;
#endif
	regsPtr->csr = S16D_RESET;
	/* enable byte swapping on input/output. We do this using
	   ioctls because the config register state is stored by
	   the driver. */
	if (ioctl (link, S16D_GET_CONFIG, &ushort) < 0)
	 { sysErrorMsg (EConnect, fxnName, "ioctl S16D_GET_CONFIG");
	   return (-1);
	 }
	ushort |= S16D_SWAP;
	if (ioctl (link, S16D_SET_CONFIG, &ushort) < 0)
	 { sysErrorMsg (EConnect, fxnName, "ioctl S16D_SET_CONFIG");
	   return (-1);
	 }
	return (link);
}

int S16DCloseLink( int link )
{
    if( link < 0 || close( link ) < 0 )
	return( -1 );
    return( 1 );
}

int S16DWriteLink( int link, void *_buf, unsigned len, int timeout )
{
	int timeoutCnt = timeout*500000;
	unsigned char *bp;
	unsigned char *bpEnd;
	volatile u_short *csrp;
	volatile u_short *datap;

	csrp = &regsPtr->csr;
	datap = &regsPtr->data;
	bp = _buf;
	bpEnd = bp + len;

	*csrp = S16D_DIR;
	while (!(*csrp & S16D_STATA_S))
	 { if (--timeoutCnt <= 0)
	    { errno = ETIMEDOUT;
	      return (0);
	    }
	 }
	while (bp < bpEnd)
	 { *datap = *bp++;
	   *csrp = (S16D_DIR | S16D_FNCT2);
	   *csrp = (S16D_DIR);
	   while (!(*csrp & S16D_STATA_S))
	    { if (--timeoutCnt <= 0)
	       { errno = ETIMEDOUT;
		 return (bp - (unsigned char*)_buf - 1);
	       }
	    }
	 }
	return (len);
}

int S16DReadLink( int link, void *_buf, unsigned len, int timeout )
{
	int cnt = 0;
	int timeoutCnt = timeout*500000;
	unsigned char *bp;
	unsigned char *bpEnd;
	volatile u_short *csrp;
	volatile u_short *datap;

	csrp = &regsPtr->csr;
	datap = &regsPtr->data;
	bp = _buf;
	bpEnd = bp + len;

	*csrp = 0;
	while (bp < bpEnd)
	 { while (!(*csrp & S16D_STATA_S))
	    { if (--timeoutCnt <= 0)
	       { errno = ETIMEDOUT;
		 return (bp - (unsigned char*)_buf);
	       }
	    }
	   *bp++ = *datap;
	 }
	return (len);
}

int S16DResetLink( int link )
{
	unsigned short cmd0;
	unsigned short cmd1;

	if (link < 0)
	 { return (-1);
	 }
	if (ioctl(link, S16D_GET_COMMAND, &cmd0) < 0)
	 { perror ("ioctl S16D_GET_COMMAND\n");
	   return (-1);
	 }
	cmd1 = cmd0 | S16D_INIT;
	if (ioctl(link, S16D_SET_COMMAND, &cmd1) < 0)
	 { perror ("ioctl S16D_SET_COMMAND\n");
	   return (-1);
	 }
	nap (0.02); 	/* wait 2 hundredths of a second */
	if (ioctl(link, S16D_SET_COMMAND, &cmd0) < 0)
	 { perror ("ioctl S16D_SET_COMMAND\n");
	   return (-1);
	 }
	return (1);
}

/* -=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=- */

static doReadLink (ld, buf, size, timeout)
int ld;
unsigned char *buf;
int size;
int timeout;
{
	int status;

	status = S16DReadLink (ld, buf, size, timeout);
	if (status != size)
	 { if (status == -1)
	    { sysErrorMsg (ETransmit, "ReadLink()", "read");
	    }
	   else
	    { errorMsg (ETransmit, "ReadLink() -- returns %d vs. %d\n",
				    status, size);
	    }
	   size = -1;
	 }
	return (size);
}

static doWriteLink (ld, buf, size, timeout)
int ld;
unsigned char *buf;
int size;
int timeout;
{
	int status;

	status = S16DWriteLink (ld, buf, size, timeout);
	if (status != size)
	 { if (status == -1)
	    { sysErrorMsg (ETransmit, "WriteLink()", "write");
	    }
	   else
	    { errorMsg (ETransmit, "WriteLink() -- returns %d vs. %d\n",
				    status, size);
	    }
	   size = -1;
	 }
	return (size);
}

s16dTspRead (dev, buf, size)
IO_DEVICE *dev;
void *buf;
int size;
{
	char *fxnName = "s16dTspRead()";
	struct s16dTspreg *s16dTspdev = (struct s16dTspreg *)dev->location;
	char headbuf[1];
#if READ_CHECKSUM
	short ourCheckSum;
	short theirCheckSum;
	unsigned char chkbuf[2];
	int i;
#endif

#if !FIXED_REPLY_SIZE 
	if (doReadLink (s16dTspdev->fd, headbuf, 1, dev->timeout) < 0)
	 { errorMsg (getErrorCode(0), "%s -- location A\n", fxnName);
	   return -1;
	 }
	if (headbuf[0])
	 { if (doReadLink (s16dTspdev->fd, buf, headbuf[0], dev->timeout) < 0)
	    { errorMsg (getErrorCode(0), 
"%s -- location B, wanted %d bytes\n", fxnName, headbuf[0]);
	      return -1;
	    }
#if READ_CHECKSUM
	   if (doReadLink (s16dTspdev->fd, chkbuf, 2, dev->timeout) < 0)
	    { errorMsg (getErrorCode(0), 
"%s -- location C, wanted %d bytes\n", fxnName, 2);
	      return -1;
	    }
	   theirCheckSum = unpackShort (chkbuf);
#endif
	 }
	dev->byteCount = headbuf[0];
#else
	 { unsigned char *bp = buf;
	   int k;

	   if (doReadLink (s16dTspdev->fd,buf,FIXED_REPLY_SIZE,dev->timeout)<0)
	    { errorMsg (getErrorCode(0), 
"%s -- location C, wanted %d bytes\n", fxnName,FIXED_REPLY_SIZE);
	      return -1;
	    }
	   dev->byteCount = *bp;
	   for (k=0; k<dev->byteCount; k++)
	    { *bp = *++bp;
	    }
#if READ_CHECKSUM
	   bp++;
	   theirCheckSum = unpackShort (bp);
#endif
	 }
#endif
#if READ_CHECKSUM
	ourCheckSum = 0;
	for (i=0; i<dev->byteCount; i++)
	 { ourCheckSum += ((unsigned char*)buf)[i];
	 }
	if (ourCheckSum != theirCheckSum)
	 { errorMsg (ETransmit, 
"%s -- bad checksum: ours=0x%x theirs=0x%x\n", ourCheckSum, theirCheckSum);
	   return (-1);
	 }
#endif	
	return (dev->byteCount);
}

s16dTspWrite (dev, buf, size)
IO_DEVICE *dev;
void *buf;
int size;
{
	/* send a buffer of 'size' bytes to the target device. Block
	   until the target device is ready to read (or we timeout). */
	   
	char *fxnName = "s16dTspWrite()";
	struct s16dTspreg *s16dTspdev = (struct s16dTspreg *)dev->location;
#if WRITE_CHECKSUM
	short checkSum;
	unsigned char chkbuf[2];
	int i;

	checkSum = 0;
	for (i=0; i<size; i++)
	 { checkSum += ((unsigned char *)buf)[i];
	 }
	packShort (checkSum, chkbuf);
#endif
	if (doWriteLink (s16dTspdev->fd, buf, size, dev->timeout) < 0)
	 { errorMsg (getErrorCode(0), "%s -- location A\n", fxnName);
	   return -1;
	 }
#if WRITE_CHECKSUM
	if (doWriteLink (s16dTspdev->fd, chkbuf, 2, dev->timeout) < 0)
	 { errorMsg (getErrorCode(0), "%s -- location B\n", fxnName);
	   return -1;
	 }
#endif
	return (size);
}

s16dTspAttach(dev)
IO_DEVICE *dev;
{
	struct s16dTspreg *s16dTspdev;
	int fd;

	if (dev->location)
	 { s16dTspdev = (struct s16dTspreg *)dev->location;
	   if (s16dTspdev->fd != -1)
	    { S16DCloseLink(s16dTspdev->fd);
	    }
	   free (s16dTspdev);
	 }
	if ((fd = S16DOpenLink(dev->addrStr)) < 0)
	 { errorMsg (EConnect, "s16dTspAttach() --\n");
	   return (-1);
	 }
	s16dTspdev = ALLOC(struct s16dTspreg);
	s16dTspdev->fd = fd;
	dev->location = (unsigned char *)s16dTspdev;
	dev->timeout = DEFAULT_TIMEOUT;

	return (1);
}

s16dTspDetach (dev)
IO_DEVICE *dev;
{
	struct s16dTspreg *s16dTspdev;

	if (dev->location)
	 { s16dTspdev = (struct s16dTspreg *)dev->location;
	   if (s16dTspdev->fd != -1)
	    { S16DCloseLink (s16dTspdev->fd);
	    }
	   free (s16dTspdev);
	   dev->location = 0;
	   return (1);
	 }
	else
	 { return (0);
	 }
}

s16dTspSignal (dev, n)
IO_DEVICE *dev;
int n;
{
	struct s16dTspreg *s16dTspdev = (struct s16dTspreg *)dev->location;

	/* Just arm the interrupts by writing and clear FNCT3
	   in the command register */

	regsPtr->csr = S16D_FNCT3; 
	regsPtr->csr = 0;
}

s16dTspIoctl (dev, com, data)
IO_DEVICE *dev;
int com;
void *data;
{
	char *fxnName = "s16dTspIoctl()";

	/* Controls various I/O parameters and also allows interrupts to
	   be enabled or disabled */

	struct s16dTspreg *s16dTspdev = (struct s16dTspreg *)dev->location;

	switch (com)
	 { case IOC_SIGINTER_ENABLE:
	    { break;
	    }
	   case IOC_SIGINTER_DISABLE:
	    { break;
	    }
	   case IOC_TIMEOUT:
	    { dev->timeout = *(int *)data;
	      break;
	    }
	   case IOC_GET_BLOCKTIME:
	    { *(int *)data = dev->timeout;
	      break;
	    }
	   case IOC_SET_BLOCKTIME:
	    { dev->timeout = *(int *)data;
	      break;
	    }
	   default:
	    { errorMsg (EBadIoctl, driIoctlMsg, fxnName, com);
	      return (-1);
	    }
	 }
	return (1);
}

s16dTspTestRead (dev)
IO_DEVICE *dev;
{
	volatile u_short *csrp;

	csrp = &regsPtr->csr;
	*csrp = 0;
	return (*csrp & S16D_STATA_S);
}

s16dTspTestWrite (dev)
IO_DEVICE *dev;
{
	volatile u_short *csrp;

	csrp = &regsPtr->csr;
	*csrp = S16D_DIR;
	return (*csrp & S16D_STATA_S);
}

#endif
