/*-=-=-=-=-=-=-=-=-=-=-=- RCCL/RCI Notice of Copyright -=-=-=-=-=-=-=-=-=-=-*
 *									    *
 *	Copyright (C) 1987, 1991 by John E. Lloyd.			    *
 *									    *
 *	Information on copyright permissions, as well as a complete	    *
 *	disclaimer of warranty, is given in the file COPYRIGHT.		    *
 *									    *
 *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/


#include "../h/cdefs.h"
#include "../h/IOdevice.h"
#include "../h/errorCodes.h"
#include "../h/errorStack.h"
#include "../h/s16dTspDevice.h"
#include "../h/rciTask.h"

#include "../drivers/driErrorMsgs.h"

#define DEFAULT_TIMEOUT		10 /* seconds */
#define FIXED_REPLY_SIZE	100

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <EDTs16d/s16d.h>
#include <EDTs16d/drv_ioctl.h> 
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/file.h>

#include <errno.h>

#define LINKNAME	"/dev/s16d0"

static testrc = 0;
static unsigned short *dataPtr;
static unsigned short *cmdPtr;
static struct s16d_regs *regsPtr;

struct s16d_regs {              /* registers 				*/
	u_short  csr ;	 	/* command/status register 		*/
	u_short  config ;	/* configuration register               */
	u_short  dummy1 ;
	u_short  count ;	/* word count 				*/
				/* set count to 1 less than desired	*/
	u_int	 dmaaddr ;	/* DMA address low			*/
	u_int    lastaddr ;	/* last address low			*/
				/* address of last word to xfer 	*/
	u_short  data;		/* Data register			*/
	u_short  dummy2;	/* for alignment			*/
};

int S16DOpenLink( char *name )
{
	int link;
	unsigned short ushort;
	unsigned int uint;

	/* Stuff for mapping the beast into memory */
	unsigned long regAddr = 0;
	int regSize = 20;
	int pageSize;
	off_t pageStart;
	caddr_t vpageStart;
	u_long pageOffset;

	if (!name)
	 { name = LINKNAME;
	 }
	else if (strncmp(name, LINKNAME, sizeof(LINKNAME)-2 ) != 0 )
	 { return (-1001);
	 }
	link = open (name, O_RDWR);

	if (link < 0 )
	 { return (-1002);
	 }

	ushort = 100; /* 1 second timeout */
	if (ioctl (link, S16S_RTIMEOUT, &ushort) < 0 )
	 { perror ("ioctl S16S_RTIMEOUT");
	   goto ioctl_failed;
	 }
	if (ioctl (link, S16S_WTIMEOUT, &ushort) < 0 )
	 { perror ("ioctl S16S_WTIMEOUT");
	   goto ioctl_failed;
	 }
	if (ioctl (link, S16G_RTIMEOUT, &ushort) < 0 )
	 { perror ("ioctl S16G_RTIMEOUT");
	   goto ioctl_failed;
	 }
	printf ("read timeout = %d\n", ushort);
	if (ioctl (link, S16G_WTIMEOUT, &ushort) < 0 )
	 { perror ("ioctl S16G_WTIMEOUT");
	   goto ioctl_failed;
	 }
	printf ("write timeout = %d\n", ushort);

	/* pulse FUNCT1 to make sure the interface logic is reset.
	   Also, issue a reset command to the board. */
	ushort = S16D_RESET | S16D_FNCT1;
	if (ioctl (link, S16S_COMMAND, &ushort) < 0 )
	 { perror ("ioctl S16S_COMMAND");
	   goto ioctl_failed;
	 }
	ushort = S16D_RESET;
	if (ioctl (link, S16S_COMMAND, &ushort) < 0 )
	 { perror ("ioctl S16S_COMMAND");
	   goto ioctl_failed;
	 }

	/* Set the read and write command words. */
	ushort = S16D_FNCT2;
	if (ioctl (link, S16S_READ_COMMAND, &ushort) < 0 )
	 { perror ("ioctl S16S_READ_COMMAND");
	   goto ioctl_failed;
	 }
	ushort = S16D_DIR | S16D_FNCT2;
	if (ioctl (link, S16S_WRITE_COMMAND, &ushort) < 0 )
	 { perror ("ioctl S16S_WRITE_COMMAND");
	   goto ioctl_failed;
	 }
	ushort = S16D_DBMODE | S16D_SWAP;
	if (ioctl (link, S16S_CONFIG, &ushort) < 0 )
	 { perror ("ioctl S16S_CONFIG");
	   goto ioctl_failed;
	 }

	 { char *tmp = (char*)getenv("S16DTESTRC");
	   if (tmp)
	    { testrc = atoi(tmp);
	    }
	 }

	/* Now map the thing into memory */

#ifdef __STDC__
    	pageSize = sysconf(_SC_PAGESIZE);
#else
    	pageSize = getpagesize();
#endif
	pageStart = (regAddr / pageSize) * pageSize;
	pageOffset = regAddr % pageSize;
	vpageStart = mmap((caddr_t)0, regSize, PROT_READ|PROT_WRITE,
			   MAP_SHARED, link, pageStart);
	if (vpageStart == (caddr_t)(-1))
	 { perror ("mmap");
	   close (link);
	   return (-1003);
	 }

	regsPtr = (struct s16d_regs *)(vpageStart + pageOffset);
	dataPtr = &regsPtr->data;
	cmdPtr = &regsPtr->csr;

	return (link);
ioctl_failed:
	close (link);
	return (-1003);
}

int S16DCloseLink( int link )
{
    if( link < 0 || close( link ) < 0 )
	return( -1 );
    return( 1 );
}

#define BUFSIZE	0x10000

int S16DWriteLink( int link, void *_buf, unsigned len, int timeout )
{
	short sbuf[BUFSIZE];
	int nsent = 0;
	int timeoutCnt = 0;
	int k, n, status;
	unsigned char *bp;
	unsigned short ushort;

	bp = _buf;
	do
	 { n = MIN(len-nsent, BUFSIZE);
	   for (k=0; k<n; k++)
	    { sbuf[k] = *bp++;
	    }
	   k = 0;
	   do
	    { status = write (link, sbuf+k, 2*(n-k));
	      if (status < 0)
	       { if (errno == ETIMEDOUT)
		  { timeoutCnt++;
		  }
		 if (errno != ETIMEDOUT || timeoutCnt == timeout)
		  { return( nsent+k ? nsent+k : status );
		  }
	       }
	      k += status/2;
	    }
	   while (k < n);
	   nsent += n;
	 }
	while (nsent < len);

	/* BIG HACK HERE: the s16d buffers its output. But for
	   proper transputer operation, S16DWriteLink should
	   block until that buffer is empty. We have arranged
	   for STATA to be true when the buffer is empty --
	   so we block on that. */

#define NUM_STRAIGHT_TRIES	5
	for (k=0, ushort=0; !(ushort & S16D_STATA_S); k++)
	 { if (k >= NUM_STRAIGHT_TRIES)
	    { if (k >= NUM_STRAIGHT_TRIES + 100*timeout - 1)
	       { errno = ETIMEDOUT;
		 return (-1);
	       }
	      nap (0.01);
	    }
	   ioctl (link, S16G_STATUS, &ushort);
	 }
	return (len);
}

int S16DReadLink( int link, void *_buf, unsigned len, int timeout )
{
	short sbuf[BUFSIZE];
	int nread = 0;
	int timeoutCnt = 0;
	int k, n, status;
	unsigned char *bp;
	unsigned short ushort;

	bp = _buf;
	do
	 { n = MIN(len-nread, BUFSIZE);
	   k = 0;
	   do
	    { status = read (link, sbuf+k, 2*(n-k));
	      if (status < 0 )
	       { if (errno == ETIMEDOUT)
		  { timeoutCnt++;
		  }
		 if (errno != ETIMEDOUT || timeoutCnt == timeout)
		  { return( nread+k ? nread+k : status );
		  }
	       }
	      k += status/2;
	    }
	   while (k < n);
	   for (k=0; k<n; k++)
	    { *bp++ = sbuf[k];
	    }
	   nread += n;
	 }
	while (nread < len);

	return (len);
}

int S16DResetLink( int link )
{
	unsigned short ushort;

	if (link < 0)
	 { return (-1);
	 }
	ushort = 2; /* hundredths of a second */
	if (ioctl(link, S16S_SEND_INIT_PULSE, &ushort) < 0)
	 { perror ("ioctl S16S_SEND_INIT_PULSE\n");
	   return (-1);
	 }
	return (1);
}

/* -=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=- */

static doReadLink (ld, buf, size, timeout)
int ld;
unsigned char *buf;
int size;
int timeout;
{
	int status;

	rciSaveContext();
	status = S16DReadLink (ld, buf, size, timeout);
	rciRestoreContext();
	if (status != size)
	 { if (status == -1)
	    { sysErrorMsg (ETransmit, "ReadLink", "");
	    }
	   else
	    { errorMsg (ETransmit, "ReadLink() -- returns %d vs. %d\n",
				    status, size);
	    }
	   size = -1;
	 }
	return (size);
}

static doWriteLink (ld, buf, size, timeout)
int ld;
unsigned char *buf;
int size;
int timeout;
{
	int status;

	rciSaveContext();
	status = S16DWriteLink (ld, buf, size, timeout);
	rciRestoreContext();
	if (status != size)
	 { if (status == -1)
	    { sysErrorMsg (ETransmit, "WriteLink", "");
	    }
	   else
	    { errorMsg (ETransmit, "WriteLink() -- returns %d vs. %d\n",
				    status, size);
	    }
	   size = -1;
	 }
	return (size);
}

s16dTspRead (dev, buf, size)
IO_DEVICE *dev;
void *buf;
int size;
{
	char *fxnName = "s16dTspRead()";
	struct s16dTspreg *s16dTspdev = (struct s16dTspreg *)dev->location;
	char headbuf[1];

	*cmdPtr = S16D_FNCT2;
		
#if !FIXED_REPLY_SIZE 
	if (doReadLink (s16dTspdev->fd, headbuf, 1, dev->timeout) < 0)
	 { errorMsg (getErrorCode(0), "%s -- location A\n", fxnName);
	   return -1;
	 }
	if (headbuf[0])
	 { if (doReadLink (s16dTspdev->fd, buf, headbuf[0], dev->timeout) < 0)
	    { errorMsg (getErrorCode(0), 
"%s -- location B, wanted %d bytes\n", fxnName, headbuf[0]);
	      return -1;
	    }
	 }
	dev->byteCount = headbuf[0];
#else
	 { unsigned char *bp = buf;
	   int k;

	   if (doReadLink (s16dTspdev->fd,buf,FIXED_REPLY_SIZE,dev->timeout)<0)
	    { errorMsg (getErrorCode(0), 
"%s -- location C, wanted %d bytes\n", fxnName, headbuf[0]);
	      return -1;
	    }
	   dev->byteCount = *bp;
	   for (k=0; k<dev->byteCount; k++)
	    { *bp = *++bp;
	    }
	 }
#endif
	*cmdPtr = 0;

	return (dev->byteCount);
}

s16dTspWrite (dev, buf, size)
IO_DEVICE *dev;
void *buf;
int size;
{
	/* send a buffer of 'size' bytes to the target device. Block
	   until the target device is ready to read (or we timeout). */
	   
	char *fxnName = "s16dTspWrite()";
	struct s16dTspreg *s16dTspdev = (struct s16dTspreg *)dev->location;

	*cmdPtr = S16D_FNCT2;

	if (doWriteLink (s16dTspdev->fd, buf, size, dev->timeout) < 0)
	 { errorMsg (getErrorCode(0), "%s -- location A\n", fxnName);
	   return -1;
	 }

	*cmdPtr = 0;

	return (size);
}

s16dTspAttach(dev)
IO_DEVICE *dev;
{
	struct s16dTspreg *s16dTspdev;
	int fd;

	if (dev->location)
	 { s16dTspdev = (struct s16dTspreg *)dev->location;
	   if (s16dTspdev->fd != -1)
	    { S16DCloseLink(s16dTspdev->fd);
	    }
	   free (s16dTspdev);
	 }
	if ((fd = S16DOpenLink(dev->addrStr)) < 0)
	 { sysErrorMsg (EConnect, "s16dTspAttach", "open");
	   return (-1);
	 }
	s16dTspdev = ALLOC(struct s16dTspreg);
	s16dTspdev->fd = fd;
	dev->location = (unsigned char *)s16dTspdev;
	dev->timeout = DEFAULT_TIMEOUT;
	return (1);
}

s16dTspDetach (dev)
IO_DEVICE *dev;
{
	struct s16dTspreg *s16dTspdev;

	if (dev->location)
	 { s16dTspdev = (struct s16dTspreg *)dev->location;
	   if (s16dTspdev->fd != -1)
	    { S16DCloseLink (s16dTspdev->fd);
	    }
	   free (s16dTspdev);
	   dev->location = 0;
	   return (1);
	 }
	else
	 { return (0);
	 }
}

s16dTspIoctl (dev, com, data)
IO_DEVICE *dev;
int com;
void *data;
{
	char *fxnName = "s16dTspIoctl()";

	/* Controls various I/O parameters and also allows interrupts to
	   be enabled or disabled */

	struct s16dTspreg *s16dTspdev = (struct s16dTspreg *)dev->location;

	switch (com)
	 { case IOC_SIGINTER_ENABLE:
	    { break;
	    }
	   case IOC_SIGINTER_DISABLE:
	    { break;
	    }
	   case IOC_TIMEOUT:
	    { dev->timeout = *(int *)data;
	      break;
	    }
	   case IOC_GET_BLOCKTIME:
	    { *(int *)data = dev->timeout;
	      break;
	    }
	   case IOC_SET_BLOCKTIME:
	    { dev->timeout = *(int *)data;
	      break;
	    }
	   default:
	    { errorMsg (EBadIoctl, driIoctlMsg, fxnName, com);
	      return (-1);
	    }
	 }
	return (1);
}

s16dTspTestRead (dev)
IO_DEVICE *dev;
{
	rciAbort (ENotSupported, "s16dTspTestRead() -- not implemented\n");
	return (0);
}

s16dTspTestWrite (dev)
IO_DEVICE *dev;
{
	rciAbort (ENotSupported, "s16dTspTestWrite() -- not implemented\n");
	return (0);
}
