#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <errno.h>
#include <unistd.h>
#include <sys/time.h>

#include <s16dreg.h>
#include <s16dioctl.h>

#define LINKNAME	"/dev/RCIs16dmem0"

static S16dRegs *regsPtr;

void nap (double sec);


int S16DOpenLink( char *name )
{
	char *fxnName = "S16DOpenLink()";
	int link;
	unsigned short ushort;
	unsigned int uint;
	volatile int delayCnt;

	/* Stuff for mapping the beast into memory */
	unsigned long regAddr = 0;
	int regSize = 20;
	int pageSize;
	off_t pageStart;
	caddr_t vpageStart;
	u_long pageOffset;

	if (!name)
	 { name = LINKNAME;
	 }
	link = open (name, O_RDWR);
	if (link < 0 )
	 { perror ("open");
	   return (-1);
	 }

	/* Now map the thing into memory */

#ifdef __STDC__
    	pageSize = sysconf(_SC_PAGESIZE);
#else
    	pageSize = getpagesize();
#endif
	pageStart = (regAddr / pageSize) * pageSize;
	pageOffset = regAddr % pageSize;
	vpageStart = mmap((caddr_t)0, regSize, PROT_READ|PROT_WRITE,
			   MAP_SHARED, link, pageStart);
	if (vpageStart == (caddr_t)(-1))
	 { perror ("mmap");
	   return (-1);
	 }

	regsPtr = (S16dRegs *)(vpageStart + pageOffset);

	/* pulse FUNCT1 to make sure the interface logic is reset.
	   Also, issue a reset command to the board. */
	regsPtr->csr = S16D_RESET | S16D_FNCT1;
	for (delayCnt=0; delayCnt<100; delayCnt++)
	   ;
	regsPtr->csr = S16D_RESET;
	/* enable byte swapping on input/output. We do this using
	   ioctls because the config register state is stored by
	   the driver. */
	if (ioctl (link, S16D_GET_CONFIG, &ushort) < 0)
	 { perror ("ioctl S16D_GET_CONFIG");
	   return (-1);
	 }
	ushort |= S16D_SWAP;
	if (ioctl (link, S16D_SET_CONFIG, &ushort) < 0)
	 { perror ("ioctl S16D_SET_CONFIG");
	   return (-1);
	 }
	return (link);
}

int S16DCloseLink( int link )
{
    if( link < 0 || close( link ) < 0 )
	return( -1 );
    return( 1 );
}

int S16DWriteLink( int link, void *_buf, unsigned len, int timeout )
{
	int timeoutCnt = timeout*500000;
	unsigned char *bp;
	unsigned char *bpEnd;
	volatile u_short *csrp;
	volatile u_short *datap;

	csrp = &regsPtr->csr;
	datap = &regsPtr->data;
	bp = _buf;
	bpEnd = bp + len;

	*csrp = S16D_DIR;
	while (!(*csrp & S16D_STATA_S))
	 { if (--timeoutCnt <= 0)
	    { errno = ETIMEDOUT;
	      return (0);
	    }
	 }
	while (bp < bpEnd)
	 { *datap = *bp++;
	   *csrp = (S16D_DIR | S16D_FNCT2);
	   *csrp = (S16D_DIR);
	   while (!(*csrp & S16D_STATA_S))
	    { if (--timeoutCnt <= 0)
	       { errno = ETIMEDOUT;
		 return (bp - (unsigned char*)_buf);
	       }
	    }
	 }
	return (len);
}

int S16DReadLink( int link, void *_buf, unsigned len, int timeout )
{
	int cnt = 0;
	int timeoutCnt = timeout*500000;
	unsigned char *bp;
	unsigned char *bpEnd;
	volatile u_short *csrp;
	volatile u_short *datap;

	csrp = &regsPtr->csr;
	datap = &regsPtr->data;
	bp = _buf;
	bpEnd = bp + len;

	*csrp = 0;
	while (bp < bpEnd)
	 { while (!(*csrp & S16D_STATA_S))
	    { if (--timeoutCnt <= 0)
	       { errno = ETIMEDOUT;
		 return (bp - (unsigned char*)_buf);
	       }
	    }
	   *bp++ = *datap;
	 }
	return (len);
}

int S16DResetLink( int link )
{
	unsigned short cmd0;
	unsigned short cmd1;

	if (link < 0)
	 { return (-1);
	 }
	if (ioctl(link, S16D_GET_COMMAND, &cmd0) < 0)
	 { perror ("ioctl S16D_GET_COMMAND\n");
	   return (-1);
	 }
	cmd1 = cmd0 | S16D_INIT;
	if (ioctl(link, S16D_SET_COMMAND, &cmd1) < 0)
	 { perror ("ioctl S16D_SET_COMMAND\n");
	   return (-1);
	 }
	nap (0.02); 	/* wait 2 hundredths of a second */
	if (ioctl(link, S16D_SET_COMMAND, &cmd0) < 0)
	 { perror ("ioctl S16D_SET_COMMAND\n");
	   return (-1);
	 }
	return (1);
}


void nap (double sec)
{
 struct timeval tv;
 struct timeval now;
 struct timeval then;
 fd_set fds;
 int n;
 extern int errno;

 gettimeofday(&now,0);
 tv.tv_sec = (int) sec;
 tv.tv_usec = (int) (1000000 * (sec - tv.tv_sec));
 then.tv_sec = now.tv_sec + tv.tv_sec;
 then.tv_usec = now.tv_usec + tv.tv_usec;
 while (then.tv_usec >= 1000000)
  { then.tv_usec -= 1000000;
    then.tv_sec ++;
  }
 while (1)
  { FD_ZERO (&fds);
    errno = 0;
    n = select(32,&fds,0,0,&tv);
    gettimeofday(&now,0);
    tv.tv_sec = then.tv_sec - now.tv_sec;
    tv.tv_usec = then.tv_usec - now.tv_usec;
    while (tv.tv_usec < 0)
     { tv.tv_usec += 1000000;
       tv.tv_sec --;
     }
    if (tv.tv_sec < 0)
     { return;
     }
  }
}
