#ifdef X86sol_TARGET
#include "../h/cdefs.h"
#include "../h/IOdevice.h"
#include <mc8prim.h>
#include <gantry_io.h>
#include <pid.h>
#include "../h/moper_com.h"
#include "../h/errorCodes.h"
#include "../h/errorStack.h"

#define PID_HEADER_BASE	0x25000

#define GANTRY_CHAN0	0
#define GANTRY_CHAN1	1
#define GANTRY_CHAN2	2

void static doclose (IO_DEVICE *dev, int numj)
{
	int k;

	fro (k=0; k<numj; k++)
	 { if (dev->args[k] != -1)
	    { close (dev->args[k]);
	    }
	 }
}

int gantry_attach(IO_DEVICE *dev)
{
	char exopenDevName[256];
	char *fxnName = "gantry_attach()";
	char *cp;
	int fd[3];
	mc8_struct *pmc8Info;

	/* open the exclusive-open devices for each of the three
	   channels */
	strcpy (exopenDevName, dev->addrStr);
	strcat (exopenDevName, "chanx");
	cp = exopenDevName+strlen(exopenDevName)-1;

	dev->args[0] = -1;
	dev->args[1] = -1;
	dev->args[2] = -1;

	*cp = GANTRY_CHAN0 + '0';
	if ((fd[0] = open (exopenDevName, O_RDWR)) == -1)
	 { sysErrorMsg (EConnect, fxnName, "open chan 0");
	   return -1;
	 }
	*cp = GANTRY_CHAN1 + '0';
	if ((fd[1] = open (exopenDevName, O_RDWR)) == -1)
	 { sysErrorMsg (EConnect, fxnName, "open chan 1");
	   doclose (dev);
	   return -1;
	 }
	*cp = GANTRY_CHAN2 + '0';
	if ((fd[2] = open (exopenDevName, O_RDWR)) == -1)
	 { sysErrorMsg (EConnect, fxnName, "open chan 2");
	   doclose (dev);
	   return -1;
	 }

	/* Do the ioctl here to enable user mode access to I/O space.
	   We can use any of the open file descriptors. */
	if (ioctl (fd[0], PMCIOC_ENABLE_IOPORTS, 1) == -1 ||
	    ioctl (fd[0], PMCIOC_ENABLE_IOPORTS, 0) == -1)
	 { sysErrorMsg (EConnect, fxnName, "ioctl PMCIOC_ENABLE_IOPORTS");
	   doclose (dev)
	   return -1;
	 }

	pmc8Info = mc8Open(dev->addr, 0, 0);
	if (pmc8Info == NULL)
	 { errorMsg (EConnect, fxnName, "mc8Open: no response from PMC\n");
	   doclose (dev);
	   return -1;
	 }
	dev->location = pmc8Info;	
	return 1;
}

void gantry_detach (IO_DEVICE *dev)
{
	mc8Close ((mc8_struct *)dev->location);
	doclose (dev);
}

void gantry_set_rate (IO_DEVICE *dev, int rate)
{
}

void gantry_set_power (IO_DEVICE *dev, int power)
{
}

void gantry_set_stop (IO_DEVICE *dev, int j)
{
}

void gantry_set_stop_all(IO_DEVICE *dev)
{
}

void gantry_set_enc (IO_DEVICE *dev, int val, int j)
{
	mc8_struct *pmc8 = (mc8_struct*)dev->location;
	
	mc8WriteMem (pmc8, PID_HEADER_BASE+(&head.channel[j].goal-&head),
		     (short*)val, 2, /*wIsInst=*/0);
}

int gantry_get_state(IO_DEVICE *dev)
{
	return IS_CALIBRATED | POWER_ON;	
}

void gantry_get_enc (IO_DEVICE *dev, int *enc)
{
	mc8_struct *pmc8 = (mc8_struct*)dev->location;
	PIDHEADER head;
	int j;

	for (j=0; j<3; j++)
	 { mc8ReadMem (pmc8, PID_HEADER_BASE+(&head.channel[j].posn-&head),
			     &enc[j], 2);
	 }
}
#endif

