#ifndef _ALLOCMEM_H_
#define _ALLOCMEM_H_

/*-=-=-=-=-=-=-=-=-=-=-=- RCCL/RCI Notice of Copyright -=-=-=-=-=-=-=-=-=-=-*
 *									    *
 *	Copyright (C) 1991 by John E. Lloyd.				    *
 *									    *
 *	Information on copyright permissions, as well as a complete	    *
 *	disclaimer of warranty, is given in the file COPYRIGHT.		    *
 *									    *
 *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

/*---------------------------------------------------------------------------
 *
 * DESCRIPTION:	Definitons for the RCI allocMem() package.
 * 
 * AUTHOR/DATE:	John Lloyd, Fall 1988
 *
 * MODIFICATION HISTORY:
 *
 *--------------------------------------------------------------------------*/

#include "../h/cdefs.h"
#include "../h/mutex.h"
#include "../h/protoDef.h"

#ifdef __cplusplus
extern "C"
{
#endif

#if defined(X86sol_TARGET)
#define MEM_POOL_SIZE	0x100000	/* size of memory allocation pool */
#else
#define MEM_POOL_SIZE	 0x80000
#endif

#define ALLOC_MEM(n,type,pool)  ((type*)allocMem(n,sizeof(type),pool))

typedef struct _MEM_HEADER {
	/* Header information attached to every mem object (but
	   hidden from view). Used internally by the system. */

	  char *name;
	  MUTEX mutex;
	  int size;	  
	  int cpu;
	  double torstenWasHere; /* alignment hack */
} MEM_HEADER;

/* Compatibility definitions for the old 'object' package */

#define allocObject(n,s,p)	allocMem(n,s,p)
#define freeObject(o)		freeMem(o)
#define findObject(a)		getMemByAddr(a)
#define getObject(n)		getMemByName(n)
#define readObject(o,b)		readMem(o,b)
#define writeObject(o,b)	writeMem(o,b)
#define accessObject(o,f,t,s)	accessMem(o,f,t,s)
#define getObjectName(o)	getMemName(o)
#define ALLOC_OBJECT(n,t,p)	ALLOC_MEM(n,t,p)

extern void *allocMem PROTO((const char*, int, int));
extern freeMem PROTO((void*));
extern void *getMemByName PROTO((const char*));
extern void *getMemByAddr PROTO((void*));
extern readMem PROTO((void*, void*));
extern writeMem PROTO((void*, void*));
extern accessMem PROTO((void*, void*, void*, int));
extern char *getMemName PROTO((void*));
extern lockMem PROTO((void*));
extern unlockMem PROTO((void*));

#ifdef __cplusplus
}
#endif

#endif
