#ifndef _GRF4_H_
#define _GRF4_H_

#include <stdio.h>
#include "../h/robotTypes.h"
#include <sys/types.h>		/* for grf4Select() prototype */
#include <sys/time.h>
#include "../h/protoDef.h"
#include "../h/varargInc.h"

#ifdef HAS_SYSSELECT_H
#include <sys/select.h>
#endif

#ifdef __cplusplus
extern "C"
{
#endif

#ifndef HLS
#define HLS	1	/* Hidden Line Support */
#endif

#define GRF_COLOR	  0x010
#define GRF_LIGHTING	  0x020
#define GRF_HARDWARE_A	  0x100
#define GRF_HARDWARE_B	  0x200

#define GRF_IDRAW_PS	  0x10000
#define GRF_FITIMAGE_PS	  0x20000

#define GRF_VIEW_CONTROL  0x1
#define GRF_X11	  	  0x2
#define GRF_WIRE_FRAME	  0x4
#define GRF_CONTROL_PANEL 0x8	/* this is set automatically */
#define GRF_2DONLY	  0x10	/* advice that it is OK to not use zbuffer */
#define GRF_DOUBLEBUF_OFF 0x20	/* suppress double buffering */

/* Button control */

#define BF_INACTIVE	0x1
#define BF_NOTBOXED	0x2
#define BF_ILLUMINATED	0x4
#define BF_SAMEROW	0x8

/* Button action codes */

#define BF_INFOCUS	0x01
#define BF_OUTFOCUS	0x02
#define BF_PRESSED	0x04
#define BF_RELEASED	0x08

#define RGBPACK(r,g,b)	(((unsigned)(b)<<16) + ((unsigned)(g)<<8) + \
			  (unsigned)(r))

#define GRF_WHITE	0xffffff
#define GRF_BLACK	0
#define GRF_RED		0xff
#define GRF_BLUE	0xff0000
#define GRF_GREEN	0xff00
#define GRF_YELLOW	(GRF_RED|GRF_GREEN)
#define GRF_CYAN	(GRF_BLUE|GRF_GREEN)
#define GRF_MAGENTA	(GRF_RED|GRF_BLUE)
#define GRF_GRAY	0x808080

/* XXX *if* we havn't defined these RCCL geometry types yet, then do so.
   There *really* out to be a better way to do this, but not yet. */

#ifndef _TRSF_H_

typedef struct {		/* Basic spatial 3-vector		*/
	float x, y, z;
} VECT, *VECT_PTR;

typedef struct {		/* 4X4 homogenous transform:		*/
	VECT n, o, a, p;	/*   the 4 column vectors of the matrix	*/
} TRSF, *TRSF_PTR;

TRSF *multTrsf();
TRSF *rpyToTrsf();
TRSF *multTrsfRpy();
TRSF *xyzToTrsf();
TRSF *multTrsfXyz();
TRSF *identTrsf();
TRSF *invertTrsf();

#endif 

typedef struct materialDef {
	unsigned long diffuse;
	unsigned long ambient;
	unsigned long emission;
	unsigned long specular;
	float shine;
	float alpha;
	  } MATERIAL_DEF;

typedef struct _cedge {
	struct _cedge *next;
	struct _cedge *prev;

	float x_b, y_b;		/* coordinates on bottom face */
	float x_t, y_t;		/* coordinates on top face */
	int hard;		/* is top-bottom connecting edge visible? */
	float *faceNrml;
	float *vrtxNrml1;
	float *vrtxNrml2;
	float *eNrml;		/* face normal turned into eye coordinates */

	float ex_b, ey_b, ez_b;
	float ex_t, ey_t, ez_t;
	float sx_b, sy_b;
	float sx_t, sy_t;
	int visible;
	  } CEDGE;

typedef struct {
	CEDGE *edges;		/* counter-clockwise list of edges */
	int size;		/* number of edges */
	float depth;		/* distance between top and bottom faces */
	int parallel;		/* top and bottom coordinates are the same */
	int topVisible;
	int bottomVisible;
} PRISM;

typedef struct {
	float *edges;		/* counter-clockwise list of edges */
	int size;
} POLYGON;

typedef struct {
	float *lines;		/* list of lines */
	int num;
} LINESET;

typedef struct {
	int npts;
	float radius;
	void* material;
	int nTheta;
	int nPhi;
	float (*pnts)[3];
  	float (*pntNormals)[3];
	float (*ctrNormals)[3];
} SPHERE;

typedef struct {
	int type;

#define GRF_PRISM	1
#define GRF_DISC	2
#define GRF_CONE	3
#define GRF_SPHERE	4
#define GRF_POLYGON	5
#define GRF_LINESET	6

	union {
	   PRISM *prism;
	   SPHERE *sphere;
	   POLYGON *polygon;
	   LINESET *lineSet;
	} ptr;

	void* material;

	TRSF trsf;
	float x1,y1,z1, x2,y2,z2; /* bounding box */

} Grf4Object;

void grf4PlaceWindow PROTO((int,int,int,int));
void grf4SetMinSize PROTO((int,int));
int grf4SetDisplay PROTO((int));
int grf4Init PROTO((char*,int,int,int));
long grf4Capacity PROTO(());
void grf4HideLines PROTO((int));
void grf4DumpPostScript PROTO((FILE*,char*,int,double,double));
void grf4PostScriptInit PROTO((FILE*,char*,int,double,double));
void grf4PostScriptFinish PROTO((FILE*));
void grf4PostScriptBoxSet PROTO((double,double,double,double));
void grf4PostScriptXform PROTO((float*,float*,float*,float*));
void grf4GetWindowSize PROTO((int*,int*));
void grf4SetDrawFxn PROTO((void(*)(void),void*));
void grf4Window PROTO((double,double,double,double,double,double,double));
void grf4Ortho PROTO((double,double,double,double,double,double,double));
void grf4DesiredWindow PROTO((float*,float*,float*,
			      float*,float*,float*,float*));
void grf4Color PROTO((int));
void grf4Clear PROTO(());
void grf4Flush PROTO(());
void grf4SetView PROTO((double,double,double,double,double,double));
void grf4SetViewTrsf PROTO((TRSF*));
void grf4PushTrans PROTO(());
void grf4PopTrans PROTO(());
void grf4ResetTrans PROTO(());
void grf4SetTrans PROTO((TRSF*));
TRSF *grf4GetTrans PROTO((TRSF*));
void grf4MultTrans PROTO((TRSF*));
void grf4Translate PROTO((double,double,double));
void grf4Rotate PROTO((double,double,double));
TRSF *grf4GetView PROTO((TRSF*));
void grf4Move PROTO((double,double,double));
void grf4Draw PROTO((double,double,double));
void grf4Point PROTO((double,double,double));
void grf4Move2 PROTO((double,double));
void grf4Draw2 PROTO((double,double));
void grf4Point2 PROTO((double,double));
void grf4PutText PROTO((char*,int,int));
void grf4GetTextExtent PROTO((int*,int*,int*,int*,char*));
Grf4Object *grf4GenDisc PROTO((int,double,double,void*));
Grf4Object *grf4GenCone PROTO((int,double,double,double,void*));
Grf4Object *grf4GenPrism PROTO((int,float*,float*,char*,double,void*));
Grf4Object *grf4GenBox PROTO((double,double,double,void*));
Grf4Object *grf4GenSphere PROTO((int,double,void*));
Grf4Object *grf4GenPolygon PROTO((int,float*,void*));
Grf4Object *grf4GenRectangle PROTO((double,double,void*));
Grf4Object *grf4GenRegPolygon PROTO((int,double,void*));
Grf4Object *grf4GenCircle PROTO((int,double,void*));
Grf4Object *grf4GenLineSet PROTO((int,float*,void*));
double grf4SetPrismDepth PROTO((Grf4Object*,double));
void grf4SetObjMaterial PROTO((Grf4Object*,void*));
void grf4DrawObject PROTO((Grf4Object*));
int grf4AddButton PROTO((char*,int,int,int(*)(int, int),long));
void grf4SetButtonFlags PROTO((int,int));
int grf4GetButtonFlags PROTO((int));
void grf4SetButtonName PROTO((int,char*));
void *grf4DefMaterialD PROTO((MATERIAL_DEF*));
void *grf4DefMaterial PROTO((unsigned long,unsigned long,unsigned long,
			     unsigned long,double,double));
void grf4SetMaterial PROTO((void*));
void grf4PushMaterial PROTO(());
void grf4PopMaterial PROTO(());
void *grf4DefLight PROTO((unsigned long,unsigned long,
			  double,double,double,double));
void grf4SetLight PROTO((int,void*));
void grf4Fatal VA_PROTO((char *fmt, ...));

double glSetDepth();

typedef struct {
	int code;
	int val1;
	int val2;
	double vx1;
	double vx2;
	int win;
} Grf4Event;

/* Fixed devices */

#define GRF_EVENTQSIZE	      128
#define GRF_NUM_EVENTS	      512

#define GRF_NULLEVENT		0

/* basic keyboard. */

#define GRF_KB_BASE		1

#define GRF_KB_A		1
#define GRF_KB_B		2
#define GRF_KB_C		3
#define GRF_KB_D		4
#define GRF_KB_E		5
#define GRF_KB_F		6
#define GRF_KB_G		7
#define GRF_KB_H		8
#define GRF_KB_I		9
#define GRF_KB_J	       10
#define GRF_KB_K	       11
#define GRF_KB_L	       12
#define GRF_KB_M	       13
#define GRF_KB_N	       14
#define GRF_KB_O	       15
#define GRF_KB_P	       16
#define GRF_KB_Q	       17
#define GRF_KB_R	       18
#define GRF_KB_S	       19
#define GRF_KB_T	       20
#define GRF_KB_U	       21
#define GRF_KB_V	       22
#define GRF_KB_W	       23
#define GRF_KB_X	       24
#define GRF_KB_Y	       25
#define GRF_KB_Z	       26
#define GRF_KB_0	       27
#define GRF_KB_1	       28
#define GRF_KB_2	       29
#define GRF_KB_3	       30
#define GRF_KB_4	       31
#define GRF_KB_5	       32
#define GRF_KB_6	       33
#define GRF_KB_7	       34
#define GRF_KB_8	       35
#define GRF_KB_9	       36
#define GRF_KB_Break	       37
#define GRF_KB_Ctrl	       39
#define GRF_KB_LeftCtrl	       GRF_KB_Ctrl
#define GRF_KB_CapsLock	       41
#define GRF_KB_RightShift      42
#define GRF_KB_LeftShift       43
#define GRF_KB_Esc	       45
#define GRF_KB_Tab	       46
#define GRF_KB_Ret	       47
#define GRF_KB_Space	       48
#define GRF_KB_Linefeed	       49
#define GRF_KB_Backspace       50
#define GRF_KB_Del	       51
#define GRF_KB_Semicolon       52
#define GRF_KB_Period	       53
#define GRF_KB_Comma	       54
#define GRF_KB_Quote	       55
#define GRF_KB_Backquote     56
#define GRF_KB_Minus	       57
#define GRF_KB_Slash	       58
#define GRF_KB_Backslash       59
#define GRF_KB_Equal	       60
#define GRF_KB_LeftBracket     61
#define GRF_KB_RightBracket    62
#define GRF_KB_LeftArrow       63
#define GRF_KB_DownArrow       64
#define GRF_KB_RightArrow      65
#define GRF_KB_UpArrow	       66
#define GRF_KB_Pad0	       67
#define GRF_KB_Pad1	       68
#define GRF_KB_Pad2	       69
#define GRF_KB_Pad3	       70
#define GRF_KB_Pad4	       71
#define GRF_KB_Pad5	       72
#define GRF_KB_Pad6	       73
#define GRF_KB_Pad7	       74
#define GRF_KB_Pad8	       75
#define GRF_KB_Pad9	       76
#define GRF_KB_PadF1	       77
#define GRF_KB_PadF2	       78
#define GRF_KB_PadF3	       79
#define GRF_KB_PadF4	       80
#define GRF_KB_PadPeriod       81
#define GRF_KB_PadMinus	       82
#define GRF_KB_PadComma	       83
#define GRF_KB_PadEnter	       84

/* the extended keyboard */

#define GRF_KB_LeftAlt	       85
#define GRF_KB_RightAlt	       86
#define GRF_KB_RightCtrl       87
#define GRF_KB_F1	       88
#define GRF_KB_F2	       89
#define GRF_KB_F3	       90
#define GRF_KB_F4	       91
#define GRF_KB_F5	       92
#define GRF_KB_F6	       93
#define GRF_KB_F7	       94
#define GRF_KB_F8	       95
#define GRF_KB_F9	       96
#define GRF_KB_F10	       97
#define GRF_KB_F11	       98
#define GRF_KB_F12	       99
#define GRF_KB_PrintScreen    100
#define GRF_KB_ScrollLock     101
#define GRF_KB_Pause	      102
#define GRF_KB_Insert	      103
#define GRF_KB_Home	      104
#define GRF_KB_PageUp	      105
#define GRF_KB_End	      106
#define GRF_KB_PageDown	      107
#define GRF_KB_NumLock	      108
#define GRF_KB_PadDivide      109
#define GRF_KB_PadMultiply    110
#define GRF_KB_PadPlus	      111

/* buttons */

#define GRF_BUTTON_BASE	      120

#define GRF_LEFT_MOUSE	      120
#define GRF_MIDDLE_MOUSE      121
#define GRF_RIGHT_MOUSE	      122

/* valuators */

#define GRF_VALUATOR_BASE     150

#define GRF_CURSOR	      150

/* miscellaneous */

#define GRF_MISC_BASE	      200

#define GRF_REDRAW	      200
#define GRF_RESIZE	      201
#define GRF_FOCUS_CHANGE      202

#define GRF_KEY_EVENT		1
#define GRF_BUTTON_EVENT	2
#define GRF_MOTION_EVENT	3
#define GRF_REDRAW_EVENT	4
#define GRF_RESIZE_EVENT	5
#define GRF_FOCUS_EVENT		6

#define GRF_MAX_EVENT_TYPES	6

#define GRF_KEY_MASK		(1<<GRF_KEY_EVENT)
#define GRF_BUTTON_MASK		(1<<GRF_BUTTON_EVENT)
#define GRF_MOTION_MASK		(1<<GRF_MOTION_EVENT)
#define GRF_REDRAW_MASK		(1<<GRF_REDRAW_EVENT)
#define GRF_RESIZE_MASK		(1<<GRF_RESIZE_EVENT)
#define GRF_FOCUS_MASK		(1<<GRF_FOCUS_EVENT)

#define GRF_EV_RESIZE		

#define GRF_IS_KEY(n)	   (GRF_KB_BASE <= (n) && (n) < GRF_BUTTON_BASE)
#define GRF_IS_BUTTON(n)   (GRF_BUTTON_BASE <= (n) && (n) < GRF_VALUATOR_BASE)
#define GRF_IS_VALUATOR(n) (GRF_VALUATOR_BASE <= (n) && (n) < GRF_MISC_BASE)
#define GRF_IS_MISC(n)     (GRF_MISC_BASE <= (n))

/* used by grf4CycleSelect */
#define GRF_GRAPHICS_EVENT	0x1
#define GRF_TIMER_EVENT		0x2

void grf4Wait PROTO((int));
int grf4Select PROTO((int*, fd_set*, fd_set*, fd_set*, struct timeval*));
void grf4Cycle PROTO(());
int grf4QueueEvent PROTO((int));
int grf4AttachEvent PROTO((int, int(*)(Grf4Event*,int), int));
int grf4ReleaseEvent PROTO((int));
int grf4PollEvent PROTO((int, Grf4Event*));
int grf4ReadEvent PROTO((Grf4Event*));
void grf4CycleWait PROTO((int, struct timeval *, int (*)()));
int grf4CycleSelect PROTO((int*, fd_set*, fd_set*, fd_set*,struct timeval*));

#ifdef __cplusplus
}
#endif

#endif
