/*-=-=-=-=-=-=-=-=-=-=-=- RCCL/RCI Notice of Copyright -=-=-=-=-=-=-=-=-=-=-*
 *									    *
 *	Copyright (C) 1987, 1991 by Mike Parker.			    *
 *									    *
 *	Information on copyright permissions, as well as a complete	    *
 *	disclaimer of warranty, is given in the file COPYRIGHT.		    *
 *									    *
 *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

#include <stdio.h>
#include <signal.h>

#ifndef Vxw_TARGET
#include <sys/types.h>
#include <sys/time.h>
#else
#include <taskLib.h>
#include <ioLib.h>
#define main test_
#endif

#include "../h/ctree.h"

char *tree;
char input[256];

SIGHAN_RETURN handler(sig)
int sig;
{
	printf ("\nhi there\n");
}

SIGHAN_RETURN onAlarm(sig)
int sig;
{
#ifndef HAS_BSD_SIGNALS
	signal (SIGALRM, onAlarm);
#endif
}

#ifdef Vxw_TARGET
int parentTaskId;
killtask()
{
	taskDelay(100);
	printf ("\nsignaling\n");
	kill (parentTaskId,SIGHUP);
}
#endif

char **argvec;

main(argc, argv)
int argc;
char **argv;
{
	int noBlocking = 0;
	int useTimer = 0;
	int status;
	int tfd = 0;

	argvec = argv;
	argc--; argv++;
	while (argc > 0)
	 { if (strcmp (*argv, "-nb") == 0)
	    { noBlocking = 1;
	    }
	   else if (strcmp (*argv, "-timer") == 0)
	    { argc--; argv++;
	      if (argc < 1)
	       { goto usage;
	       }
	      useTimer = atol(*argv);
	    }
	   else
	    { goto usage;
	    }
	   argc--; argv++;
	   continue;
usage:
	   fprintf(stderr,"Usage: %s [-nb] [-timer <msec>]\n", argvec[0]);
	   exit(-1);
	 }

	tree = tree_match_parse("\
	(quit								\
	  debug (on off)						\
	  forget (memoryAlloc						\
		  memoryFoobar						\
	 	 )							\
	show								\
	importMemory							\
)",0);
	if (! tree)
	 { fprintf(stderr,"Parse failed: %.30s\n",tree_match_err_at());
	   exit(1);
	 }
	input[0] = '\0';
	signal(SIGHUP, handler);
	printf ("pid=%d\n", getpid());
#if 0 /* Vxw_TARGET */
	parentTaskId = taskIdSelf();
	taskSpawn ("", 1, (VX_SUPERVISOR_MODE|VX_DEALLOC_STACK|VX_STDIO),
		       0x1000, killtask);
#endif

	if (useTimer)
	 { struct itimerval itval;

	   itval.it_interval.tv_sec = useTimer/1000;
	   itval.it_interval.tv_usec = (useTimer%1000)*1000;
	   itval.it_value.tv_sec = useTimer/1000;
	   itval.it_value.tv_usec = (useTimer%1000)*1000;

	   signal (SIGALRM, onAlarm);
	   setitimer (ITIMER_REAL, &itval, (struct itimerval*)0);
	   tree_match_sig_catch_off (SIGALRM);
	 }
	if (noBlocking)
	 { tree_match_setup();
	   while (strcmp(input,"quit"))
	    { 
	      status = tree_match_nb (tree,"BigLongPrompt> ",input,&tfd);
	      if (status == 1)
	       { 
		 printf("%s\n",input);
		 if (!strcmp(input,"debug on"))
		  { *tree_match_flags() |= CTREE_F_DEBUG;
		  }
		 else if (!strcmp(input,"debug off"))
		  { *tree_match_flags() &= ~CTREE_F_DEBUG;
		  }
		 else if (!strcmp(input,"show"))
		  { printf ("show what?\n");
		  }
		 else if (!strcmp(input,"importMemory"))
		  { FILE *inf;
		    int i;
		    inf = tree_match_input_stream();
		    if (fscanf(inf,"%d",&i) != 1)
		     { printf("fscanf error");
		     }
		    else
		     { printf("number is %d\n",i);
		     }
		  }
		 continue;
	       }
#ifdef Vxw_TARGET
	       { int n = 0;
		 do
		  { ioctl(tfd, FIONREAD, &n);
		  }
		 while (n <= 0);
	       }
#else
	       { fd_set rset;
		 FD_ZERO(&rset);
		 FD_SET(tfd,&rset);
		 if (select (FD_SETSIZE, &rset, (fd_set*)0, (fd_set*)0,
						 (struct timeval*)0) <= 0)
		  { fprintf(stderr, "error -- select returned with nothing\n");
		    tree_match_reset();
		    exit (-1);
		  }
	       }
#endif
	    }
	   tree_match_reset();
	 }
	else
	 { while (strcmp(input,"quit"))
	    { tree_match(tree,"BigLongPrompt> ",input);
	      printf("%s\n",input);
	      if (!strcmp(input,"debug on"))
	       { *tree_match_flags() |= CTREE_F_DEBUG;
	       }
	      else if (!strcmp(input,"debug off"))
	       { *tree_match_flags() &= ~CTREE_F_DEBUG;
	       }
	      else if (!strcmp(input,"show"))
	       { printf ("show what?\n");
	       }
	      else if (!strcmp(input,"importMemory"))
	       { FILE *inf;
		 int i;
		 inf = tree_match_input_stream();
		 if (fscanf(inf,"%d",&i) != 1)
		  { printf("fscanf error");
		  }
		 else
		  { printf("number is %d\n",i);
		  }
	       }
	    }
	 }
}
