/*-=-=-=-=-=-=-=-=-=-=-=- RCCL/RCI Notice of Copyright -=-=-=-=-=-=-=-=-=-=-*
 *									    *
 *	Copyright (C) 1987, 1991 by John E. Lloyd and Vincent Hayward.	    *
 *									    *
 *	Information on copyright permissions, as well as a complete	    *
 *	disclaimer of warranty, is given in the file COPYRIGHT.		    *
 *									    *
 *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

/*=~=~=~=~ McGill-Computer-Vision-and-Robotics-Laboratory--C-Module ~=~=~=~=
 *		
 * MODULE NAME:	linearJacob.c
 *
 * DESCRIPTION:	Jacobian computations for the Stanford arm.
 *		For description, see 'man pumaJacob'
 * 
 * AUTHOR/DATE:	John Lloyd, Spring 1987, after Vincent Hayward, Purdue, 1983
 *		Rewrote routines to interface with new RCI, winter 1988.
 *
 *~=~=~=~=~=~= McGill-Computer-Vision-and-Robotics-Laboratory ~=~=~=~=~=~=*/

#include <math.h>

#include "../h/fastmath.h"
#include "../h/cdefs.h"
#include "../h/robot_jls.h"
#include "../h/robotMath.h"
#include "../h/linear_jls.h"
#include "../h/linear_kynvar.h"
#include "../h/robotTypes.h"

#define EPSILONF 2.0e-6

#define J1	0

/* ---- Definition of var structure parameters ---- */

linearFwdJacobT(t, f, j6, kyn)
float *t;
FORCE *f;
float *j6;
KYN *kyn;
{
	LINEAR_VAR var;

	linearUpdateVar (&var, j6, kyn);
	return (linearFwdJacobTVar (t, f, &var, kyn));
}

linearFwdJacob(dc, dj, j6, kyn)
DIFF *dc;
float *dj;
float *j6;
KYN *kyn;
{
	LINEAR_VAR var;

	linearUpdateVar (&var, j6, kyn);
	return (linearFwdJacobVar (dc, dj, &var, kyn));
}

linearInvJacobT(f, t, j6, kyn, epsilon)
FORCE *f;
float *t;
float *j6;
KYN *kyn;
double epsilon;
{
	LINEAR_VAR var;

	linearUpdateVar (&var, j6, kyn);
	return (linearInvJacobTVar (f, t, &var, kyn, epsilon));
}

linearInvJacob(dj, dc, j6, kyn, epsilon)
float *dj;
DIFF *dc;
float *j6;
KYN *kyn;
double epsilon;
{
	LINEAR_VAR var;

	linearUpdateVar (&var, j6, kyn);
	return (linearInvJacobVar (dj, dc, &var, kyn, epsilon));
}

linearFwdJacobTVar(j, f, var, kyn) /*::*/
float *j;
FORCE *f;
void *var;
KYN *kyn;
{
	register LINEAR_VAR *svar = (LINEAR_VAR*)var;

	j[J1] = f->f.x;
}

linearFwdJacobVar(d, q, var, kyn) /*::*/
register DIFF_PTR d;
register float *q;
void *var;
KYN *kyn;
{
	register LINEAR_VAR *svar = (LINEAR_VAR*)var;

	d->t.x = q[J1];
	d->t.y = 0;
	d->t.z = 0;
	d->r.x = 0;
	d->r.y = 0;
	d->r.z = 0;
}

linearInvJacobVar(q, d, var, kyn, epsilon)
register float *q;
register DIFF_PTR d;
void *var;
KYN *kyn;
double epsilon;
{
	LINEAR_KYN *skyn = (LINEAR_KYN*)kyn->ext;
	register LINEAR_VAR *svar = (LINEAR_VAR*)var;

	q[J1] = d->t.x;
	return (0);
}

linearInvJacobTVar(f, t, var, kyn, epsilon)
register FORCE *f;
register float *t;
void *var;
KYN *kyn;
double epsilon;
{
	LINEAR_KYN *skyn = (LINEAR_KYN*)kyn->ext;
	register LINEAR_VAR *svar = (LINEAR_VAR*)var;

	f->f.x = t[J1];
	f->f.y = 0;
	f->f.z = 0;
	f->m.x = 0;
	f->m.y = 0;
	f->m.z = 0;
	return (0);
}

/* Jacobian updating functions */

linearUpdateVar (var, ang, kyn)
void *var;
register float *ang;
KYN *kyn;
{
	register LINEAR_VAR *svar = (LINEAR_VAR*)var;

	linearUpdateVarXsincos (var, ang, kyn);
}

linearUpdateVarXsincos (var, ang, kyn)
void *var;
float *ang;
KYN *kyn;
{
   
	register LINEAR_KYN *skyn = (LINEAR_KYN*)kyn->ext;
	register LINEAR_VAR *svar = (LINEAR_VAR*)var;

	svar->d[J1] = ang[J1];
}
