/*-=-=-=-=-=-=-=-=-=-=-=- RCCL/RCI Notice of Copyright -=-=-=-=-=-=-=-=-=-=-*
 *									    *
 *	Copyright (C) 1987, 1991 by John E. Lloyd and Vincent Hayward.	    *
 *									    *
 *	Information on copyright permissions, as well as a complete	    *
 *	disclaimer of warranty, is given in the file COPYRIGHT.		    *
 *									    *
 *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

/*=~=~=~=~ McGill-Computer-Vision-and-Robotics-Laboratory--C-Module ~=~=~=~=
 *		
 * MODULE NAME:	linearKinematic.c
 *
 * DESCRIPTION:	Forward and inverse kinematic routines for the LINEAR robots.
 * 
 * AUTHOR/DATE:	John Lloyd, Spring 1987, after Vincent Hayward, Purdue, 1983
 *		Rewrote routines to interface with new RCI, winter 1988.
 *
 *~=~=~=~=~=~= McGill-Computer-Vision-and-Robotics-Laboratory ~=~=~=~=~=~=*/

#include <math.h>

#include "../h/fastmath.h"
#include "../h/cdefs.h"
#include "../h/robot_jls.h"
#include "../h/robotMath.h"
#include "../h/linear_jls.h"
#include "../h/linear_kynvar.h"
#include "../h/robotTypes.h"

#include "../h/rcclDefs.h"	/* for F_UNDEF */

#define EPSILONF 2.0e-6

/* performs mod 2 pi */

static float range(a) /*##*/	/* Sets a to  0 <= a < pi2  */
double a;
{
	if (a > 15. || a < -15.)
	 { return(10.);
	 }

	while(a >= PIT2)
	 { a -= PIT2;
	 }
	while(a < 0.)
	 { a += PIT2;
	 }
	if (a > PIT2)
	 { return(0.);
	 }
	return(a);
}

/* ---- Definition of link parameters to make code more readable ---- */

#define J1	0

linearFwdKinematics (t, conf, j, kyn, var)
TRSF *t;	  	  /* resulting transform */
unsigned long *conf;	  /* returns configuration information */
float *j;		  /* joints in range coordinates + configuration */
KYN *kyn;	  	  /* kinematic parameters */
void *var;		  /* palce for storing computing sines/cosines */
{
	register LINEAR_VAR *svar = (LINEAR_VAR*)var;
	GEN_KYN *gen = kyn->gen;

	t->n.x = 1;
	t->n.y = 0;
	t->n.z = 0;

	t->o.x = 0;
	t->o.y = 1;
	t->o.z = 0;

	t->a.x = 0;
	t->a.y = 0;
	t->a.z = 1;

	t->p.x = j[J1];
	t->p.y = 0;
	t->p.z = 0;

	if (conf != NULL)
	 { *conf = 0;
	 }

	if (var != NULL)	/* Update sines and cosinesd if needed */
	 { svar->d[J1] = j[J1];
	 }
}

linearInvKinematics(j, t, conf, refj, kyn, jls, var) /*::*/
float *j;	  	  /* joints in range coordinates + configuration */
TRSF *t;	  /* input transform 			  	 */
unsigned long conf;	  /* configuration information			 */
float *refj;		  /* reference joints				 */
KYN *kyn;		  /* kinematic parameters			 */
void *var;		  /* run-time variables 			 */
JLS *jls;		  /* joint level data -- usd to get min. joints	 */
{
	LINEAR_KYN *skyn = (LINEAR_KYN*)kyn->ext;
	register LINEAR_VAR *svar = (LINEAR_VAR*)var;
	GEN_KYN *gen = kyn->gen;
	TRSF TR;
	int code = 0;

	float ang[LINEARJ];			/* Non-ranged angle values */

	ang[J1] = t->p.x;

	code |= linearRangeAngles (j, ang, refj, kyn, jls);

	if (var != NULL)	/* Update sines and cosines if needed */
	 { svar->d[J1] = j[J1];
	 }
	return(code);
}

linearRangeAngles (r, a, refj, kyn, jls)
register float *r, *a;
float *refj;
KYN *kyn;
JLS *jls;
{
	register float *jmin, *jmax;
	register double angle;
	register double refAngle;
	int code = 0;

	jmin = jls->jmin;
	jmax = jls->jmax;

	if (a[J1] <= jmin[J1] || a[J1] >= jmax[J1])
	 { code |= (1 << 0);
	 }
	r[J1] = a[J1];
	return (code);
}

linearSolveConf(conf, j, kyn)
unsigned long *conf;	/* configuration description */
register float *j;	/* joints in radians */
KYN *kyn;
{
	*conf = 0;
}
