/* Functions which implement the RCI control level on Sun4 Solaris */

#if defined(Sun4sol_TARGET) || defined(X86sol_TARGET)

#include <stdio.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/priocntl.h>
#include <sys/rtpriocntl.h>
#include <sys/tspriocntl.h>
#include <sys/lock.h>
#include <sys/mman.h>
#include <sys/lwp.h>

#include "rcisys.h"
#include "rciioctl.h"
#include "rciTask.h"

#include "../h/cdefs.h"
#include "../h/errorCodes.h"
#include "../h/errorStack.h"

#include "../rci/sun4solCtrlTask.h"

#define CTRL_STACK_SIZE	0x40000

/* ----- Error Messages. Defined externally to save space ----- */

static char *badIoctlMsg	= "%s -- %s ioctl failed, errno %d\n";
static char *noResponseMsg	= "%s -- can't signal control task\n";

int rciIsOpen;
RciProcId rciCtrlTaskId;
RciProcId rciMainTaskId;
lwpid_t rciCtrlTaskLwp;
lwpid_t rciMainTaskLwp;
int rciClockFd;
int rciPlanCtrlFds[2];
int rciCtrlPlanFds[2];

FILE *rciStdout;

fd_set intrDevFds;
IntrDevDesc *intrDevList = NULL;

extern int rciSignalRefCount;
extern int rciSimOnly;

static int (*alarmFxn)();
static int alarmArg;
static int alarmTime;

static int (*beatFxn)();
static int beatArg;
static int beatPeriod;

static int (*execFxn)();
static int execArg;

static growStack();

static SIGHAN_RETURN onXcpu(sig)
int sig;
{
	/* Called when the planning level exits and sends the
	   control level an XCPU message. */

	exit (-1);
}

int pbset;
int first;

char rciPrintfBuf[1024];
char *rciPrintfBufp;
extern FILE *rciStdout;

static execCtrlFxn(fxn, arg)
int (*fxn)();
int arg;
{
	int status;
	volatile int spin;

	pbset = 1;
	rciPrintfBufp = rciPrintfBuf;
	(*fxn)(arg);	
	pbset = 0;
	if (rciPrintfBufp != rciPrintfBuf)
	 { fprintf (rciStdout, "%s", rciPrintfBuf);
	   fflush (rciStdout);
	   rciPrintfBufp = rciPrintfBuf;
	 }
}

id_t schedinfo (name, maxpri)
char *name;
int *maxpri;
{
	pcinfo_t info;
	rtinfo_t *rtinfop;
	tsinfo_t *tsinfop;
	int nc;

	(void) strcpy (info.pc_clname, name);
	nc = priocntl (0L, 0L, PC_GETCID, (caddr_t)&info);

	if (nc == -1L)
	 { return (-1);
	 }
	if (strcmp ("RT", name) == 0)
	 { rtinfop = (rtinfo_t*)&info.pc_clinfo;
	   if (maxpri)
	    { *maxpri = rtinfop->rt_maxpri;
	    }
	 }
	else if (strcmp ("TS", name) == 0)
	 { tsinfop = (tsinfo_t*)&info.pc_clinfo;
	   if (maxpri)
	    { *maxpri = tsinfop->ts_maxupri;
	    }
	 }
	else
	 { errno = EINVAL;
	   return (-1);
	 }
	return (info.pc_cid);
}

static id_t RTcid = PC_CLNULL;
static id_t TScid = PC_CLNULL;
static int maxRTpri;

rciGetMaxRTpri ()
{
	if (RTcid == PC_CLNULL)
	 { /* initialize RTcid if necessary */
	   if ((RTcid = schedinfo ("RT", &maxRTpri)) == -1)
	    { perror ("getMaxRTpri: schedinfo");
	      exit (1);
	    }
	 }
	return (maxRTpri);
}

int rciGetSchedPri (int *classp, lwpid_t id)
{
	pcparms_t parms;
	rtparms_t *rtparms;
	tsparms_t *tsparms;

	if (RTcid == PC_CLNULL)
	 { /* initialize RTcid if necessary */
	   if ((RTcid = schedinfo ("RT", &maxRTpri)) == -1)
	    { perror ("setRealTime: schedinfo");
	      exit (1);
	    }
	 }
	parms.pc_cid = PC_CLNULL;
	if (priocntl (P_LWPID, id, PC_GETPARMS, (caddr_t)&parms) == -1L)
	 { perror ("rciGetSchedPri: priocntl PC_GETPARMS");
	   exit (-1);
	 }
	if (parms.pc_cid == RTcid)
	 { if (classp) *classp = 1;
	   rtparms = (rtparms_t*)&parms.pc_clparms;
	   return (rtparms->rt_pri);
	 }
	else
	 { if (classp) *classp = 0;
	   tsparms = (tsparms_t*)&parms.pc_clparms;
	   return (tsparms->ts_upri);
	 }
}

rciSetRealTime (level, qSecs, qNanoSecs, parms)
int level, qSecs, qNanoSecs;
pcparms_t *parms;
{
	pcparms_t _parms;
	rtparms_t *rtparms;
	int status;

	if (RTcid == PC_CLNULL)
	 { /* initialize RTcid if necessary */
	   if ((RTcid = schedinfo ("RT", &maxRTpri)) == -1)
	    { perror ("setRealTime: schedinfo");
	      exit (1);
	    }
	 }
	if (parms)
	 { /* get current parameter settings into parms */
	   _parms.pc_cid = PC_CLNULL;
	   if (priocntl (P_LWPID, _lwp_self(), PC_GETPARMS,
					       (caddr_t)&_parms) == -1L)
	    { perror ("setRealTime: priocntl PC_GETPARMS");
	      exit (-1);
	    }
	   *parms = _parms;
	 }
	_parms.pc_cid = RTcid;
	rtparms = (rtparms_t*)&_parms.pc_clparms;
	rtparms->rt_pri = level;
	rtparms->rt_tqsecs = qSecs;
	rtparms->rt_tqnsecs = qNanoSecs;
#ifndef REALTIME_DISABLE
	status = priocntl (P_LWPID, _lwp_self(), PC_SETPARMS, 
			 (caddr_t)&_parms);
	if (status < 0 && rciSimOnly == 0)
	 { perror ("rciSetRealTime: priocntl PC_SETPARMS");
	   exit (1);
	 }
#endif
	return (status);
}

rciClearRealTime (parms)
pcparms_t *parms;
{
	pcparms_t *pptr;
	int status;

	if (parms)
	 { pptr = parms;
	 }
	else
	 { pcparms_t _parms;
	   tsparms_t *tsparms;

	   if (TScid == PC_CLNULL)
	    { if ((TScid = schedinfo ("TS", NULL)) == -1)
	       { perror ("clearRealTime: schedinfo");
		 exit (1);
	       }
	    }
	   _parms.pc_cid = TScid;
	   tsparms = (tsparms_t*)&_parms.pc_clparms;
	   tsparms->ts_uprilim = 0;
	   tsparms->ts_upri = 0;
	   pptr = &_parms;
	 }
#ifndef REALTIME_DISABLE
	status = priocntl (P_LWPID, _lwp_self(), PC_SETPARMS, (caddr_t)pptr);
#endif
	if (status < 0 && rciSimOnly == 0)
	 { perror ("rciClearRealTime: priocntl PC_SETPARMS");
	   exit (1);
	 }
/* begin DBG */
	 { int class;
	   int pri;

	   pri = rciGetSchedPri (&class, _lwp_self());
	   if (class == 1)
	    { 
printf ("rciClearRealTime didn't! pri=%d id=%d origclass=%d RTclass=%d\n",
		       pri, _lwp_self(), pptr->pc_cid, RTcid);
	      abort ();
	    }
	 }
/* end DBG */
	return (status);
}

int rciWatchPoint = 0;

void *rciWatchDogTask (void *arg)
{
	pcparms_t parms;
	int last, new, active;
	extern RCI_DESC rciTaskTable[];
	int k;

	if (rciSimOnly)
	 { rciSetRealTime (rciGetMaxRTpri(), 1, 0, &parms);
	 }
	last = rciWatchPoint = 1;
	while (1)
	 { nap (5.0);
	   new = rciWatchPoint;
	   active = 0;
	   for (k=0; k<MAX_RCI_TASKS; k++)
	    { RCI_TCB *tcb = rciTaskTable[k].tcb;
	      if (tcb && tcb->controlMode == ACTIVE)
	       { active = 1;
		 break;
	       }
	    }
	   if (active && new == last)
	    { printf ("RCI watchdog timeout\n");
	      abort ();
	    }
	   last = new;
	 }	
}

rciCtrlTask()
{
	fd_set rfds, intrDevSav;
	int time;
	int n, k;
	char com;

	rciCtrlTaskLwp = _lwp_self();

	/* Start up the control task, set priorites. */
#if 0
	if (plock(PROCLOCK) < 0)    /* not sure if we inherit plock() */
	 { perror("plock:");	    /* from the parent or not. */
	   exit (-1);
	 }
#endif
#if 0
  	sun4LockStack(CTRL_STACK_SIZE);
#else
	growStack();
#endif
#if USE_RAW_LWP == 0
	/* block signals */
	 { sigset_t sigs;
	   sigfillset (&sigs);
	   sigdelset (&sigs, SIGBUS);
	   sigdelset (&sigs, SIGSEGV);
	   if (thr_sigsetmask (SIG_BLOCK, &sigs, (sigset_t*)0) != 0)
	    { perror ("thr_sigsetmask");
	      exit (1);
	    }
	 }
#endif
	if (rciSimOnly == 0)
	 { rciSetRealTime (rciGetMaxRTpri()-2, 1, 0, NULL);
	 }
#if 0
	signal (SIGTSTP, SIG_IGN);
	signal (SIGINT,  SIG_IGN);
#endif
	signal (SIGXCPU, onXcpu);

/*	prctl (PR_RESIDENT); */

	alarmFxn = NULLF;
	beatFxn = NULLF;
	execFxn = NULLF;

#ifdef X86sol_TARGET
	/* On Intel PC machines, we need to explictly enable access
	   to the I/O ports from user mode. On Solaris X86 machines, 
	   we do this by setting bits 0x3000 in the EFLAGS register,
	   using a special ioctl */
	if (ioctl (rciClockFd, RCIIOC_ENABLE_IOPORTS, 1) < 0 ||
	    ioctl (rciClockFd, RCIIOC_ENABLE_IOPORTS, 0) < 0)
	 { perror ("ioctl ENABLE IOPORTS");
	   exit (1);
	 }
#endif
	nap (0.02);		    /* wait for all this to sink in */

	/* send acknowledge to main task that we are here. */

	com = RCI_CTRL_ACK;
	write (rciCtrlPlanFds[1], &com, 1);

	if (rciSimOnly == 0)
	 { 
	   if (thr_create ((void*)0, 0, rciWatchDogTask, (void*)0, 
			    THR_BOUND, (thread_t*)0) != 0)
	    { perror ("thr_create (creating watchdog)");
	      exit (1);
	    }
	   while (rciWatchPoint == 0)
	    { nap (0.01);
	    }
	 }

	while (1)
	 { register IntrDevDesc *id;

	   /* The task sits in a select polling loop, looking for

	      1. input from the clock device, which indicates clock 
		 tick expiry and the need to run a timed task;

	      2. input on any of the RCI interrupt devices, which
		 indicates an interrupt has arrived and we should
		 run in the interrupt monitor;

	      3. input from the parent task, which indicates 
		 the parent wants us to execute an RCI function.
	    */


	   do
	    { rfds = intrDevSav = intrDevFds;
	      FD_SET (rciPlanCtrlFds[0], &rfds);
	      FD_SET (rciClockFd, &rfds);

	      n = select (FD_SETSIZE, &rfds, (fd_set*)0, (fd_set*)0,
					     (struct timeval*)0);
	      if (n<0 && memcmp(&intrDevSav,&intrDevFds,sizeof(fd_set)) == 0
		      && errno != EINTR)
	       { /* we check intrDevFds because the parent may have closed
		    one of the files there, causing select to return EBADF;
		    this is OK and we should not exit then. */
		 perror ("rciCtrlTask: error in select");
		 exit (-1);
	       }
	    }
	   while (n<0);
	   rciWatchPoint++;

	   for (id=intrDevList; id; id=id->next)
	    { if (FD_ISSET(id->fd, &rfds))
	       { 
		 /* interrupt occured on device fd. */
		 if ((n = read_restart (id->fd, &com, 1)) < 0)
		  { perror ("rciCtrlTask: read from intr -- ");
		    exit (-1);
		  }
		 if (n == 0)
		  { fprintf (stderr,
"rciCtrlTask: read from intr device returns wrong size\n");
		    exit (-1);
		  }
		 if (id->fxn)
		  { execCtrlFxn (id->fxn,id->arg);
		  }
	       }
	    }
	   if (FD_ISSET(rciClockFd, &rfds))
	    { RciTimerEvent tev;

	      if ((n = read_restart (rciClockFd, (char*)&tev, sizeof(tev))) < 0)
	       { perror ("rciCtrlTask: read from clock -- ");
		 exit (-1);
	       }
	      if (n != sizeof(tev))
	       { fprintf (stderr,
"rciCtrlTask: read from clock device returns wrong size\n");
		 exit (-1);
	       }
	      if (tev.req & RCI_ALARM_READY)
	       { if (alarmFxn)
		  { execCtrlFxn (alarmFxn, alarmArg);
		  }
	       }
	      if (tev.req & RCI_BEAT_READY)
	       { if (beatFxn)
		  { execCtrlFxn (beatFxn, beatArg);
		  }
	       }
	    }
	   if (FD_ISSET(rciPlanCtrlFds[0], &rfds))
	    { /* parent task wants us to do something */

	      if ((n = read_restart (rciPlanCtrlFds[0], &com, 1)) < 0)
	       { perror ("rciCtrlTask: read from plan -- ");
		 exit (-1);
	       }
	      if (n == 0)
	       { fprintf (stderr,
"rciCtrlTask: read from plan device returns wrong size\n");
		 exit (-1);
	       }
	      switch (com)
	       { case RCI_CTRL_EXEC:
		  { if (execFxn)
		     { execCtrlFxn (execFxn, execArg);
		       execFxn = 0;
		     }
		    break;
		  }
		 default:
		  { fprintf (stderr,
"rciCtrlTask: bogus command (0x%x)from planning task\n", com);
		    exit (-1);
		  }
	       }
	      com = RCI_CTRL_ACK;
	      write (rciCtrlPlanFds[1], &com, 1);
	    }
	 }		
}

int rciSchedTime;

rciSchedAst (clock, time, fxn, arg)
int clock, time;
int (*fxn)();
int arg;
{
#if 0
extern int lastschedast_tick;
lastschedast_tick = time;
#endif

	/* This function is always called from the control task context. */
	/* clock is ignored for now */

	alarmFxn = fxn;
	alarmArg = arg;
	alarmTime = time;
	rciSchedTime = time;
	if (ioctl (rciClockFd, RCIIOC_ALARM_REQ, time) < 0)
	 { perror ("rciCtrlTask: ALARM_REQ ioctl -- ");
	   exit (-1);
	 }
}

rciClockExec (ticks, fxn, arg)
int ticks;
int (*fxn)();
int arg;
{
	/* Always called in the planning task context. */

	char *fxnName = "rciClockExec()";

#if !defined(SIMONLY_CFG)
	if (rciSimOnly)
	 { return (1);
	 }
	beatPeriod = ticks;
	beatFxn = fxn;
	beatArg = arg;
	if (ioctl (rciClockFd, RCIIOC_BEAT_REQ, ticks) < 0)
	 { errorMsg (EBadDev, badIoctlMsg, fxnName, "BEAT_REQ", errno);
	   return (-1);
	 }
#endif
	return(1);
}

rciTaskExec (fxn, arg)
int (*fxn)();
int arg;
{
	/* Always called in the planning task context. */

	char *fxnName = "rciTaskExec()";
	char com;
	int n;

	if (rciSimOnly == 1)
	 { if (rciSignalRefCount > 0)
	    { blockRciSignals();
	    }
	   (*fxn)(arg);
	   if (rciSignalRefCount > 0)
	    { unblockRciSignals();
	    }
	 }
	else
	 { 
#ifndef SIMONLY_CFG
	   execFxn = fxn;
	   execArg = arg;
	   com = RCI_CTRL_EXEC;
	   if (write (rciPlanCtrlFds[1], &com, 1) < 0 ||
		read_restart (rciCtrlPlanFds[0], &com, 1) <= 0 ||
		com != RCI_CTRL_ACK)
	    { errorMsg (EInternal, noResponseMsg, fxnName);
	      return (-1);
	    }
#endif
	 }
	return(1);
}

static growStack()
{
	char stack[CTRL_STACK_SIZE];
	int pagesize;
	char *cp;
	volatile char c;
	
	pagesize = getPageSize();
	for (cp=stack; cp<(&stack[CTRL_STACK_SIZE]); cp+=pagesize)
	 { c = *cp;
	 }
}

sun4LockStack(size)
int size;
{
	int pagesize;
	char *cp, *stackbase;
	volatile char c;

	pagesize = getPageSize();
	/* this SHOULD be PRDA->usrstk; */
	stackbase = (char*)&pagesize;
	for (cp=stackbase; cp>=stackbase-(size+pagesize)+1; cp-=pagesize)
	 { c = *cp;
	 }
#ifndef REALTIME_DISABLE
	if (mlock (stackbase-size+1, size) < 0)
	 { perror ("lockStack: mlock -- ");
	   exit (-1);
	 }
#endif
}

#else	/* !defined(Sun4sol_TARGET) && !defined(X86sol_TARGET) */
static int keepRanlibHappy;
#endif
