#include <stdio.h>
#include <math.h>
#include "../h/stringDefs.h"	
#include "../h/varargInc.h"
#include <sys/types.h>
#include <sys/time.h>
#ifdef sgi
#include <stdlib.h>
#endif

#include "../h/cdefs.h"
#include "../h/grf4.h"
#include "grf4Internal.h"

#ifndef M_PI
#define M_PI    3.14159265358979323846
#endif

#define X11_H		1
#define SUNVIEW_H	2
#define GL_H		3
#define GL_WF_H	4

#if HLS
int grf4HiddenLineMode = 0;
#endif

int grf4GeneralFlags;

static int hostType;
static int displayFlags;
static int initDone;

static int winw;
static int winh;
static int directionOfY;

static void (*redrawFxn)();
static void *redrawArg;

#define fxntab grf4FunctionTable
struct grf4fxnTable fxntab;

static float desiredLeft;
static float desiredRight;
static float desiredBottom;
static float desiredTop;
static float desiredNear;
static float desiredFar;
static float desiredAspectRatio;

static float virtL;
static float virtB;
static float virtW;
static float virtH;

static int perspective;

static WinPlaceDesc placeDesc;

BUTTON_DESC *buttonList;
int numButtons;
int rowNumber = 1;

int buttonSpacing;
int buttonWidth;
int buttonHeight;
int panelWidth;
int panelHeight;

#define NO_VIEW_CTRL		0
#define YAW_PITCH_CTRL		1
#define X_Y_CTRL		2
#define ZOOM_CTRL		3
#define PANY_PANX_CTRL		4
#define TILT_CTRL		5
	
static int viewCtrlMode = NO_VIEW_CTRL;
static int yCtrlButton;
static int xCtrlButton;
static int yawCtrlButton;
static int pitchCtrlButton;
static int panXCtrlButton;
static int panYCtrlButton;
static int tiltCtrlButton;
static int zoomCtrlButton;
static int xMotionEnable = 1;
static int yMotionEnable = 1;

#define ROLL_BUTTON	0
#define PITCH_BUTTON	1
#define YAW_BUTTON	2

static struct rpyButton {
	float val;
	int dispButton;
	int upButton;
	int downButton;
  } rpyButtons[3];

static void updateRpyButtons();

typedef struct _TRSF_BLK {
	struct _TRSF_BLK *next;
	TRSF T;
  } TRSF_BLK;

static TRSF_BLK *trsfStack;

/* Note: T is the entire transform from object coordinates to the
   window frame. E is only the portion of this transform from
   the viewing center to the window frame. In other words,

	T = E D

   where D is a transform from object coordinates to the viewing center.

   That's why when E is set, T must be reset as well, according
   to

	T = Enew Eold^-1 D
*/
static TRSF E;
static TRSF T;
static TRSF L;	/* not used right now */

static drawObjCnt = 0;

static int determineHostType();
static popMaterial();
static pushMaterial();
static adjustScreenTrans();
static BUTTON_DESC *getButton();

static double range180(a)
double a;
{
	/* puts an angle in the range -180 < a <= 180 */

	double norm;
	
	norm = a/(2*180);
	norm -= (int)norm;
	if (norm > 0.5)
	 { return (180*(2*(norm-1)));
	 }
	else if (norm <= -0.5)
	 { return (180*(2*(norm+1)));
	 }
	else
	 { return (180*(2*(norm)));
	 }
}

grf4GetDrawObjCnt()
{
	return (drawObjCnt);
}

void grf4PlaceWindow (x,y,w,h)
int x, y, w, h;
{
	placeDesc.x = x;
	placeDesc.y = y;
	placeDesc.w = w;
	placeDesc.h = h;
	placeDesc.placed = 1;
}

void grf4SetMinSize (w, h)
int w, h;
{
	placeDesc.minW = w;
	placeDesc.minH = h;
}

static int determineHostType(flags)
int flags;
{
	extern char *getenv();

	if (flags & GRF_X11)
	 {
#ifdef X11_GRAPHICS
	   if (getenv ("DISPLAY"))
	    { return (X11_H);
	    }
#endif
	   return (0);
	 }
#ifdef GL_GRAPHICS
	if (flags & GRF_WIRE_FRAME)
	 { return (GL_WF_H);
	 }
	else
	 { return (GL_H);
	 }
#else
	if (getenv("DISPLAY"))
	 { return (X11_H);
	 }
#ifdef sun
	else if (getenv("WINDOW_PARENT"))
	 { return (SUNVIEW_H);
	 }
#endif
	else
	 { return (0);
	 }
#endif
}

typedef struct matStackElem {
	struct matStackElem *next;
	void *m;
} MatStackElem;

static MatStackElem *matStackFree;
static MatStackElem *matStack;

static void* currentMaterial;

static popMaterial()
{
	MatStackElem *se;

	if (se = matStack)
	 { matStack = se->next;
	   (*fxntab.setMaterial)(se->m);
	   se->next = matStackFree;
	   matStackFree = se;
	 }
}

static pushMaterial()
{
	MatStackElem *se;

	if ((se = matStackFree) == 0)
	 { se = ALLOC(MatStackElem);
	 }
	else
	 { matStackFree = se->next;
	 }
	se->next = matStack;
	matStack = se;
	se->m = currentMaterial;
}

static int fxnInitDone;

doFxnInit(flags)
int flags;
{
	fxnInitDone = 1;

	if ((hostType = determineHostType(flags)) == 0)
	 { return (-1);
	 }

	/* instantiate functions depending on host type */

	switch (hostType)
	 { case GL_H:
	    { if (grf4GlSetupFxns (&fxntab) < 0)
	       { return (-1);
	       }
	      displayFlags = 0;
	      break;
	    }
	   case GL_WF_H:
	    { if (grf4GlwfSetupFxns (&fxntab) < 0)
	       { return (-1);
	       }
	      gdrSetupFxns (&fxntab);
	      displayFlags = GRF_WIRE_FRAME;
	      break;
	    }
	   case SUNVIEW_H:
	    { if (grf4SvSetupFxns (&fxntab) < 0)
	       { return (-1);
	       }
	      gdrSetupFxns (&fxntab);
	      displayFlags = GRF_WIRE_FRAME;
	      break;
	    }
	   case X11_H:
	    { if (grf4X11SetupFxns (&fxntab) < 0)
	       { return (-1);
	       }
	      gdrSetupFxns (&fxntab);
	      displayFlags = GRF_WIRE_FRAME | GRF_X11;
	      break;
	    }
	 }
	return (1);
}

int grf4SetDisplay (flags)
int flags;
{
	if (!fxnInitDone)
	 { if (doFxnInit(flags) < 0)
	    { return (-1);
	    }
	 }
	if (hostType == 0)
	 { return (-1);
	 }
	return (displayFlags | (flags & ~(GRF_WIRE_FRAME | GRF_X11)));
}

static void onButton();
static void onRpyButton();

int grf4Init (name, fgcolor, bgcolor, flags)
char *name;
int fgcolor;
int bgcolor;
int flags;
{
	int x,y,w,h;

	extern void buttonCompile();

	if (!fxnInitDone)
	 { if (doFxnInit(flags) < 0)
	    { return (-1);
	    }
	 }
	if (hostType == 0)
	 { return (-1);
	 }

	if (flags & GRF_VIEW_CONTROL)
	 { 
	   grf4AddButton("View Control", BF_NOTBOXED|BF_INACTIVE, 1, 0, 0);
	   yawCtrlButton = grf4AddButton("yaw", BF_ILLUMINATED, 1,
					  onButton, YAW_PITCH_CTRL);
	   pitchCtrlButton = grf4AddButton("pitch", BF_SAMEROW|BF_ILLUMINATED,
					  1, onButton, YAW_PITCH_CTRL);
	   xCtrlButton = grf4AddButton("x", 0, 1, 
					  onButton, X_Y_CTRL);
	   yCtrlButton = grf4AddButton("y", BF_SAMEROW, 1, 
					  onButton, X_Y_CTRL);
	   zoomCtrlButton = grf4AddButton("zoom", 0, 1, onButton, ZOOM_CTRL);
	   panXCtrlButton = grf4AddButton("pan x", 0, 1, 
					  onButton, PANY_PANX_CTRL);
	   panYCtrlButton = grf4AddButton("pan y", BF_SAMEROW, 1, 
					  onButton, PANY_PANX_CTRL);
	   tiltCtrlButton = grf4AddButton("tilt", 0, 1, onButton, TILT_CTRL);

	   rpyButtons[ROLL_BUTTON].upButton = 
	      grf4AddButton("+", 0, 1, onRpyButton, ROLL_BUTTON);
	   rpyButtons[ROLL_BUTTON].dispButton = 
	      grf4AddButton("roll:     0.0", BF_INACTIVE|BF_SAMEROW, 3, 0,0);
	   rpyButtons[ROLL_BUTTON].downButton = 
	      grf4AddButton("-", BF_SAMEROW, 1, onRpyButton, ROLL_BUTTON);

	   rpyButtons[PITCH_BUTTON].upButton = 
	      grf4AddButton("+", 0, 1, onRpyButton, PITCH_BUTTON);
	   rpyButtons[PITCH_BUTTON].dispButton = 
	      grf4AddButton("pitch:    0.0", BF_INACTIVE|BF_SAMEROW, 3, 0,0);
	   rpyButtons[PITCH_BUTTON].downButton = 
	      grf4AddButton("-", BF_SAMEROW, 1, onRpyButton, PITCH_BUTTON);

	   rpyButtons[YAW_BUTTON].upButton = 
	      grf4AddButton("+", 0, 1, onRpyButton, YAW_BUTTON);
	   rpyButtons[YAW_BUTTON].dispButton = 
	      grf4AddButton("yaw:      0.0", BF_INACTIVE|BF_SAMEROW, 3, 0,0);
	   rpyButtons[YAW_BUTTON].downButton = 
	      grf4AddButton("-", BF_SAMEROW, 1, onRpyButton, YAW_BUTTON);

	   viewCtrlMode = YAW_PITCH_CTRL;
	 }

	switch (hostType)
	 { case GL_H:
	    { if (grf4GlInit (name, fgcolor, bgcolor, &placeDesc, flags) < 0)
	       { fprintf (stderr, "Could not initialize GL graphics\n");
		 return (-1);
	       }
	      break;
	    }
	   case GL_WF_H:
	    { if (grf4GlwfInit (name, fgcolor, bgcolor, &placeDesc, flags) < 0)
	       { fprintf (stderr, "Could not initialize GL wireframe graphics\n");
		 return (-1);
	       }
	      break;
	    }
	   case SUNVIEW_H:
	    { if (grf4SvInit (name, fgcolor, bgcolor, &placeDesc, flags) < 0)
	       { fprintf (stderr, "Could not initialize SunView graphics\n");
		 return (-1);
	       }
	      break;
	    }
	   case X11_H:
	    { if (grf4X11Init (name, fgcolor, bgcolor, &placeDesc, flags) < 0)
	       { fprintf (stderr, "Could not initialize X11 graphics\n");
		 return (-1);
	       }
	      break;
	    }
	 }
	(*fxntab.getWindowSize) (&winw, &winh, &directionOfY);
	buttonCompile (panelWidth, panelHeight);

	if (panelWidth)
	 { flags |= GRF_CONTROL_PANEL;
	 }

	if (hostType != GL_H)
	 { gdrInit (winw, winh, directionOfY);
	 }
	(*fxntab.ortho)(-0.5, winw+0.5, -0.5, winh+0.5, -1.0, 1.0);
	identTrsf (&E);
	identTrsf (&L);	/* L not used right now */
	identTrsf (&T);
	(*fxntab.loadTrans)(&T);

	if (viewCtrlMode != NO_VIEW_CTRL)
	 { updateRpyButtons(0.0, 0.0, 0.0);
	 }

	drawAllButtons ();

	currentMaterial = (*fxntab.defMaterial)(0xffffff, 0, 0, 0, 0.0, 0.0);
	(*fxntab.setMaterial)(currentMaterial);

	grf4InitEvents(flags, winw, winh, panelWidth, panelHeight);

	initDone = 1;
	grf4GeneralFlags = flags;

	return (displayFlags | (flags & ~(GRF_WIRE_FRAME | GRF_X11)));
}

long grf4Capacity()
{
	return (*fxntab.capacity)();
}

void grf4HideLines(on)
int on;
{
	grf4HiddenLineMode = on;
}

void grf4DumpPostScript(fp,name,flags,w,h)
FILE *fp;
char *name;
int flags;
double w, h;
{
	PostScriptInit (fp, name, flags, w, h);
}

void grf4PostScriptInit (fp,name,flags,w,h)
FILE *fp;
char *name;
int flags;
double w, h;
{
	PostScriptInit (fp, name, flags, w, h);
}

void grf4PostScriptFinish (fp)
FILE *fp;
{
	PostScriptFinish (fp);
}

void grf4PostScriptBoxSet (llx, lly, urx, ury)
double llx, lly, urx, ury;
{
	PostScriptBoxSet (llx, lly, urx, ury);
}

void grf4PostScriptXform (xmp, xbp, ymp, ybp)
float *xmp, *xbp, *ymp, *ybp;
{
	PostScriptXform (xmp, xbp, ymp, ybp);
}

void grf4GetWindowSize (w, h)
int *w, *h;
{
	*w = winw;
	*h = winh;
}

void grf4SetDrawFxn (fxn, arg)
void (*fxn)();
void *arg;
{
	redrawFxn = fxn;
	redrawArg = arg;
}

void grf4Window (left, right, bottom, top, near, far, aspectSet)
double left, right, bottom, top, near, far;
double aspectSet;
{
	float winAr;

	desiredLeft = left;
	desiredRight = right;
	desiredBottom = bottom;
	desiredTop = top;
	desiredNear = near;
	desiredFar = far;
	desiredAspectRatio = aspectSet;

	perspective = 1;

	adjustScreenTrans (winw, winh);

	identTrsf (&E);
	identTrsf (&T);
	(*fxntab.loadTrans)(&T);

	if (viewCtrlMode != NO_VIEW_CTRL)
	 { updateRpyButtons(0.0, 0.0, 0.0);
	 }
}

void grf4Ortho (left, right, bottom, top, near, far, aspectSet)
double left, right, bottom, top, near, far;
double aspectSet;
{
	desiredLeft = left;
	desiredRight = right;
	desiredBottom = bottom;
	desiredTop = top;
	desiredNear = near;
	desiredFar = far;
	desiredAspectRatio = aspectSet;

	perspective = 0;

	adjustScreenTrans (winw, winh);

	identTrsf (&E);
	identTrsf (&T);
	(*fxntab.loadTrans)(&T);

	if (viewCtrlMode != NO_VIEW_CTRL)
	 { updateRpyButtons(0.0, 0.0, 0.0);
	 }
}

static adjustScreenTrans(w, h)
int w, h;
{
	/* Adjust either the width or the height so that the aspect
	   ratio defined by

	((desiredTop-desiredBottom)*w)/((desiredRight-desiredLeft)*h)

	   remains constant.
	 */

	float right   = desiredRight;
	float left    = desiredLeft;
	float bottom  = desiredBottom;
	float top     = desiredTop;
	float delta;
	float ar;

	ar = ((top-bottom)*w)/((right-left)*h);

	if (desiredAspectRatio > 0)
	 { 
	   if (ar > desiredAspectRatio)
	    { /* adjust by increasing the width */

	      delta = (ar/desiredAspectRatio - 1.0)*(right-left)/2.0;
	      right += delta;
	      left -= delta;
	    }
	   else if (ar < desiredAspectRatio)
	    { /* adjust by increasing the height */

	      delta = (desiredAspectRatio/ar - 1.0)*(top-bottom)/2.0;
	      top += delta;
	      bottom -= delta;
	    }
	    }

	if (perspective)
	 { (*fxntab.window) (left, right, bottom, top, 
			     desiredNear, desiredFar);
	 }
	else
	 { (*fxntab.ortho) (left, right, bottom, top, 
			     desiredNear, desiredFar);
	 }
	virtL = left;
	virtB = bottom;
	virtH = top - bottom;
	virtW = right - left;
}

void grf4DesiredWindow (lp, rp, bp, tp, np, fp, ap)
float *lp, *rp, *bp, *tp, *np, *fp, *ap;
{
	/* hack for internal use */

	if (lp) *lp = desiredLeft;
	if (rp) *rp = desiredRight;
	if (bp) *bp = desiredBottom;
	if (tp) *tp = desiredTop;
	if (np) *np = desiredNear;
	if (fp) *fp = desiredFar;
	if (ap) *ap = desiredAspectRatio;
}

void grf4Color (color)
int color;
{
	(*fxntab.color)(color);
}

void grf4Clear ()
{
	(*fxntab.clear)();
}

void grf4Flush ()
{
	drawObjCnt = 0;

	(*fxntab.flush)();
}

static int doSetView (Enew, updateButtons)
TRSF *Enew;
int updateButtons;
{
	(*fxntab.loadTrans)(Enew);
	(*fxntab.resetLights)();
	multLiTrsf (&T, &E, &T);
	multTrsf (&T, Enew, &T);
	(*fxntab.loadTrans)(&T);
	E = *Enew;

	if (viewCtrlMode != NO_VIEW_CTRL && updateButtons==YES)
	 { float r, p, y;

	   trsfToRpy (&r, &p, &y, &E);
	   r = range180 (r);
	   p = range180 (p);
	   y = range180 (y);

	   updateRpyButtons(r, p, y);
	 }
	return (1);
}

void grf4SetView (x,y,z, roll,pitch,yaw)
double x,y,z, roll,pitch,yaw;
{
	TRSF Enew;

	rpyToTrsf (&Enew, roll,pitch,yaw);
	Enew.p.x = x;
	Enew.p.y = y;
	Enew.p.z = z;

	updateRpyButtons (roll, pitch, yaw);
	doSetView (&Enew, /*updateButtons=*/NO);
}

void grf4SetViewTrsf (Enew)
TRSF *Enew;
{
	doSetView (Enew, /*updateButtons=*/YES);
}

void grf4PushTrans ()
{
	TRSF_BLK *tb;

	tb = (TRSF_BLK*)malloc(sizeof(TRSF_BLK));
	tb->next = trsfStack;
	trsfStack = tb;
	tb->T = T;
}

void grf4PopTrans ()
{
	TRSF_BLK *tb;

	if (tb = trsfStack)
	 { T = tb->T;
	   trsfStack = tb->next;
	   free ((char*)tb);
	   (*fxntab.loadTrans)(&T);
	 }
}

void grf4ResetTrans ()
{
	identTrsf (&T);
	(*fxntab.loadTrans)(&T);
}

void grf4SetTrans (Tnew)
TRSF *Tnew;
{
	/* Sets the 4X4 transform describing the spatial position of the
	   indicated object. 
	 */
	T = *Tnew;
	(*fxntab.loadTrans)(&T);
}

TRSF *grf4GetTrans (Tr)
TRSF *Tr;
{
	/* Gets the 4X4 transform describing the spatial position of the
	   indicated object. 
	 */
	*Tr = T;
	return (Tr);
}

void grf4MultTrans (D)
TRSF *D;
{
	multTrsf (&T, &T, D);
	(*fxntab.loadTrans)(&T);
}

void grf4Translate (x,y,z)
double x,y,z;
{
	TRSF D;

	identTrsf (&D);
	D.p.x = x;
	D.p.y = y;
	D.p.z = z;

	multTrsf (&T, &T, &D);
	(*fxntab.loadTrans)(&T);
}

void grf4Rotate (roll,pitch,yaw)
double roll,pitch,yaw;
{
	TRSF D;

	rpyToTrsf (&D, roll,pitch,yaw);
	D.p.x = 0.0;
	D.p.y = 0.0;
	D.p.z = 0.0;

	multTrsf (&T, &T, &D);
	(*fxntab.loadTrans)(&T);
}

TRSF *grf4GetView (Er)
TRSF *Er;
{
	*Er = E;
	return (Er);
}
	
void grf4Move (x, y, z)
double x, y, z;
{
	(*fxntab.move)(x, y, z);
}

void grf4Draw (x, y, z)
double x, y, z;
{
	(*fxntab.draw)(x, y, z);
}

void grf4Point (x, y, z)
double x, y, z;
{
	(*fxntab.point)(x, y, z);
}

void grf4Move2 (x, y)
double x, y;
{
	(*fxntab.move2)(x, y);
}

void grf4Draw2 (x, y)
double x, y;
{
	(*fxntab.draw2)(x, y);
}

void grf4Point2 (x, y)
double x, y;
{
	(*fxntab.point2)(x, y);
}

void grf4MoveRaw (x, y)
double x, y;
{
	(*fxntab.moveRaw)(x, y);
}

void grf4DrawRaw (x, y)
double x, y;
{
	(*fxntab.drawRaw)(x, y);
}

void grf4PointRaw (x, y)
double x, y;
{
	(*fxntab.pointRaw)(x, y);
}

void grf4PutText (str, x, y)
char *str;
int x, y;
{
	(*fxntab.putText)(str, x, y);
}

void grf4GetTextExtent (xp, yp, wp, hp, str)
int *xp, *yp, *wp, *hp;
char *str;
{
	(*fxntab.getTextExtent)(xp, yp, wp, hp, str);
}

Grf4Object *grf4GenDisc (npts, radius, depth, material)
int npts;
double radius, depth;
void *material;
{
	/* creates a disc defined by a regular polygon of 
	   'npts' in the x-y plane, projected upwards along
	   z by a distance 'depth'. The top face lies in the
	   plane z=depth, while the bottom face lies in the
	   plane z=0. The regular polygon is centered on
	   the x-y plane origin. 

	   'material' points to a grf4 material definition.
	 */

	Grf4Object *obj;

	if (!fxnInitDone)
	 { if (doFxnInit(0) < 0)
	    { return (0);
	    }
	 }
	if (hostType == 0)
	 { return (0);
	 }
	obj = (Grf4Object*)calloc(1,sizeof(Grf4Object));
	obj->ptr.prism = (*fxntab.genDisc)(npts, radius, depth);
	obj->type = GRF_PRISM;
	if (material)
	 { obj->material = material;
	 }
	return (obj);
}

Grf4Object *grf4GenCone (npts, radius1, radius2, depth, material)
int npts;
double radius1, radius2, depth;
void *material;
{
	Grf4Object *obj;

	if (!fxnInitDone)
	 { if (doFxnInit(0) < 0)
	    { return (0);
	    }
	 }
	if (hostType == 0)
	 { return (0);
	 }
	obj = (Grf4Object*)calloc(1,sizeof(Grf4Object));
	obj->ptr.prism = (*fxntab.genCone)(npts, radius1, radius2, depth);
	obj->type = GRF_PRISM;
	if (material)
	 { obj->material = material;
	 }
	return (obj);
}

Grf4Object *grf4GenPrism(npts, bot, top, hidden, depth, material)
int npts;		
float *bot, *top;
char *hidden;
double depth;
void *material;
{
	/* Creates a prism formed by extruding a polygon in the
	   x-y plane along the z coordinate by an amount given by
	   'depth'. The bottom face corresponds to z=0, while the
	   top face corresponds to z=depth.

	   'npts' is the number of points in the defining polygon.

	   'bot' is an 2*npts array of floats defining the x-y vertices
	   of the bottom polygon (x1, y1, x2, y2, ... ). The vertices
	   are assumed to be ordered counter-clockwise when viewed from
	   *inside* the prism.

	   'top' is an optional argument giving the vertices of the
	   top polygon (which may be different from bottom polygon but
	   which still must have npts vertices). If top=NULL, then 
	   the top polygon is assumed to be identical to the bottom
	   polygon.

	   'hidden' is an array of 'npts' chars which determines
	   how the side edges of the prism should be rendered. If
	   hidden[i] == '1', then the edge corresponding to the i-th
	   polygon vertex is assumed to be smooth, and should be
	   rendered using shading. Otherwise, if hidden[i] == '0',
	   the edge is assumed to be crisp. For example, all the
	   vertices of a smooth disk would have hidden[i] == '1'.

	   'material' points to a grf4 material definition.
	 */

	Grf4Object *obj;

	if (!fxnInitDone)
	 { if (doFxnInit(0) < 0)
	    { return (0);
	    }
	 }
	if (hostType == 0)
	 { return (0);
	 }
	obj = (Grf4Object*)calloc(1,sizeof(Grf4Object));
	obj->ptr.prism = (*fxntab.genPrism)(npts, bot, top, hidden, depth);
	obj->type = GRF_PRISM;
	if (material)
	 { obj->material = material;
	 }
	return (obj);
}

Grf4Object *grf4GenBox(lenx, leny, lenz, material)
double lenx, leny, lenz;
void *material;
{
	
	float bot[8];
	Grf4Object *obj;

	if (!fxnInitDone)
	 { if (doFxnInit(0) < 0)
	    { return (0);
	    }
	 }
	if (hostType == 0)
	 { return (0);
	 }
	obj = (Grf4Object*)calloc(1,sizeof(Grf4Object));

	/* generate the bottom face */

	bot[0] = 0.0;   bot[1] = 0.0;
	bot[2] = lenx;  bot[3] = 0.0;
	bot[4] = lenx;  bot[5] = leny;
	bot[6] = 0.0;   bot[7] = leny;

	obj->ptr.prism = (*fxntab.genPrism)(4, bot, (float*)0, (char*)0, lenz);
	obj->type = GRF_PRISM;
	if (material)
	 { obj->material = material;
	 }
	return (obj);
}

Grf4Object *grf4GenSphere(npts, radius, material)
int npts;
double radius;
void *material;
{
	/* Creates a sphere of given 'radius', centered on the
	   origin, using 'npts'*'npts' polygons. 'material' is
	   a GRF4 material definition. */

	Grf4Object *obj;

	if (!fxnInitDone)
	 { if (doFxnInit(0) < 0)
	    { return (0);
	    }
	 }
	if (hostType == 0)
	 { return (0);
	 }
	obj = (Grf4Object*)calloc(1,sizeof(Grf4Object));
	obj->ptr.sphere = (*fxntab.genSphere)(npts, radius);
	obj->type = GRF_SPHERE;
	if (material)
	 { obj->material = material;
	 }
	return (obj);
}

Grf4Object *grf4GenPolygon(npts, vtxs, material)
int npts;
float *vtxs;
void *material;
{
	Grf4Object *obj;

	if (!fxnInitDone)
	 { if (doFxnInit(0) < 0)
	    { return (0);
	    }
	 }
	if (hostType == 0)
	 { return (0);
	 }
	obj = (Grf4Object*)calloc(1,sizeof(Grf4Object));
	obj->ptr.polygon = (*fxntab.genPolygon)(npts, vtxs);
	obj->type = GRF_POLYGON;
	if (material)
	 { obj->material = material;
	 }
	return (obj);
}

Grf4Object *grf4GenRectangle (width, height, material)
double width, height;
void *material;
{
	float vtxs[8];

	vtxs[0] = -width/2;
	vtxs[1] = -height/2;
	vtxs[2] =  width/2;
	vtxs[3] = -height/2;
	vtxs[4] =  width/2;
	vtxs[5] =  height/2;
	vtxs[6] = -width/2;
	vtxs[7] =  height/2;
	return (grf4GenPolygon (4, vtxs, material));
}

Grf4Object *grf4GenRegPolygon(npts, size, material)
int npts;
double size;
void *material;
{
	Grf4Object *obj;

	if (!fxnInitDone)
	 { if (doFxnInit(0) < 0)
	    { return (0);
	    }
	 }
	if (hostType == 0)
	 { return (0);
	 }
	obj = (Grf4Object*)calloc(1,sizeof(Grf4Object));
	obj->ptr.polygon = (*fxntab.genRegPolygon)(npts, size);
	obj->type = GRF_POLYGON;
	if (material)
	 { obj->material = material;
	 }
	return (obj);
}

Grf4Object *grf4GenCircle (npts, radius, material)
int npts;
double radius;
void *material;
{
	/* This is the same as grf4GenRegPolygon(), except that
	   the size is controlled by a radius instead of a side length */

	double ang;	/* half the angle subtended by one side */
	double side;

	if (!fxnInitDone)
	 { if (doFxnInit(0) < 0)
	    { return (0);
	    }
	 }
	if (hostType == 0)
	 { return (0);
	 }
	ang = M_PI/npts;
	/* largest radius is given by (side/2)/sin(ang), and the
	   smallest radius is given by (side/2)/tan(ang), where 'side'
	   is the length of one side of the approximating polygon.
	   We calculate side so that radius reflects the average
	   of these two amounts. */
	side = 4*radius/(1/sin(ang) + 1/tan(ang));

	return (grf4GenRegPolygon(npts, side, material));
}

Grf4Object *grf4GenLineSet (size, lines, material)
int size;
float *lines;
void *material;
{
	Grf4Object *obj;

	if (!fxnInitDone)
	 { if (doFxnInit(0) < 0)
	    { return (0);
	    }
	 }
	if (hostType == 0)
	 { return (0);
	 }
	obj = (Grf4Object*)calloc(1,sizeof(Grf4Object));
	obj->ptr.lineSet = (*fxntab.genLineSet)(size, lines);
	obj->type = GRF_LINESET;
	if (material)
	 { obj->material = material;
	 }
	return (obj);
}

double grf4SetPrismDepth (obj, depth)
Grf4Object *obj;
double depth;
{
	if (obj->type != GRF_PRISM)
	 { return (-1.0);
	 }
	else
	 { return (*fxntab.setDepth)(obj, depth);
	 }	
}

void grf4SetObjMaterial (obj, material)
Grf4Object *obj;
void *material;
{
	obj->material = material;
}

void grf4DrawObject (obj)
Grf4Object *obj;
{
	/* Renders the indicated object */

	int matSaved = 0;

	drawObjCnt++;

	if (obj->material && obj->material != currentMaterial)
	 { matSaved = 1;
	   pushMaterial();
	   (*fxntab.setMaterial)(obj->material);
	 }
	switch (obj->type)
	 { case GRF_PRISM:
	    { (*fxntab.drawPrism)(obj);
	      break;
	    }
	   case GRF_DISC:
	    { (*fxntab.drawPrism)(obj);
	      break;
	    }
	   case GRF_CONE:
	    { (*fxntab.drawPrism)(obj);
	      break;
	    }
	   case GRF_SPHERE:
	    { (*fxntab.drawSphere)(obj);
	      break;
	    }
	   case GRF_POLYGON:
	    { (*fxntab.drawPolygon)(obj);
	      break;
	    }
	   case GRF_LINESET:
	    { (*fxntab.drawLineSet)(obj);
	      break;
	    }
	 }
	if (matSaved)
	 { popMaterial();
	 }
}

/* -------------------- button module ------------------ */

/* The button panel is laid out, horizontally, as follows:
	
	|<------ winw ------>||<-----panelWidth------>|
	XXXXXXXXXXXXXXXXXXXXXXO.....BBBBBBBBBBBBBB.....
			       |<->||<-- b->w -->|
		    buttonSpacing|

   The vertical layout looks like this:
	._
	.|
	.| buttonSpacing
	.- 
	B- 
	B| b->h
	B| 
	B- b->y
*/

static BUTTON_DESC *getButton(num)
int num;
{
	BUTTON_DESC *b;

	for (b=buttonList; b; b=b->next)
	 { if (b->num == num)
	    { return (b);
	    }
	 }
	return(0);
}

int grf4AddButton(name, flags, sizeK, fxn, arg)
char *name;
int flags;
int sizeK;
int (*fxn)();
long arg;
{
	BUTTON_DESC *b;

	if (initDone)
	 { fprintf (stderr, 
		    "grf4AddButton(): must be called before grf4Init()\n");
	   exit(-1);
	 }
	if (sizeK < 1)
	 { fprintf (stderr, "grf4AddButton(): bad value for sizeK\n");
	   exit (-1);
	 }

	b = ALLOC(BUTTON_DESC);
	b->next = buttonList;
	buttonList = b;

	b->name = (char*)malloc(strlen(name)+1);
	strcpy (b->name, name);
	b->flags = flags;
	b->fxn = fxn;
	b->arg = arg;
	b->num = ++numButtons;
	b->sizeK = sizeK;
	return (b->num);
}

void grf4SetButtonFlags (num, flags)
int num, flags;
{
	BUTTON_DESC *b;

	if (b = getButton (num))
	 { b->flags = flags;
	   if (initDone)	/* then grf4Init() has been called */
	    { (*fxntab.drawButton)(b);
	    }
	 }
}

int grf4GetButtonFlags (num)
int num;
{
	BUTTON_DESC *b;

	if (b = getButton (num))
	 { return (b->flags);
	 }
	return (-1);
}

void grf4SetButtonName (num, name)
int num;
char *name;
{
	BUTTON_DESC *b;

	if (b = getButton (num))
	 { if (b->name)
	    { free (b->name);
	    }
	   b->name = (char*)malloc(strlen(name)+1);
	   strcpy (b->name, name);
	   if (initDone)	/* then grf4Init() has been called */
	    { (*fxntab.getButtonExtent)(b);
	      (*fxntab.drawButton)(b);
	    }
	 }
}

void buttonInit(w, h)
int *w, *h;
{
	/* Called by the local init function whenever it is safe
	   to call getButtonExtent(). It returns the desired height
	   and width of the button panel. */

	BUTTON_DESC *b, *b_next, *bRowBegin;
	int rowSizeK, n, kw, kwMax, row, maxH, maxW;

	/* If there are no buttons, return a panelWidth of 0 */

	if (buttonList == 0)
	 { *w = panelWidth = 0;
	   *h = panelHeight = 0;
	   return;
	 }

	/* Reverse the direction of the button list */

	b = buttonList;
	buttonList = 0;
	for ( ; b; b=b_next)
	 { b_next = b->next;
	   b->next = buttonList;
	   buttonList = b;
	 }

	buttonSpacing = 4; /* MAX(4,0.2*maxH) */

	/* Compute the  minimum height and width for each button,
	   and row number, for each button. Compute a few width
	   parameters relevant for each row as well. */

	row = -1;
	maxW = maxH = 0;
	for (b=buttonList; b; b=b->next)
	 { if (!(b->flags & BF_SAMEROW) || row == -1)
	    { row++;
	      bRowBegin = b;
	      rowSizeK = kwMax = n = 0;
	    }
	   (*fxntab.getButtonExtent)(b);
	   if (b->minh > maxH)
	    { maxH = b->minh;
	    }
	   b->row = row;
	   rowSizeK += b->sizeK;
	   n++;
	   if ((kw = ceil(b->minw/(double)b->sizeK)) > kwMax)
	    { kwMax = kw;
	    }
	   if (!b->next || !(b->next->flags & BF_SAMEROW))
	    { bRowBegin->rowSizeK = rowSizeK;
	      bRowBegin->rowMinW = rowSizeK*kwMax + (n-1)*buttonSpacing;
	      bRowBegin->rowN = n;
	      if (bRowBegin->rowMinW > maxW)
	       { maxW = bRowBegin->rowMinW;
	       }
	    }
	 }

	buttonWidth = maxW;
	buttonHeight = maxH;
	/* the extra 1 on panelWidth is for the dividing line */
	panelWidth = maxW + 2*buttonSpacing + 1;
	panelHeight = (row+2)*buttonSpacing + (row+1)*buttonHeight;

	*w = panelWidth;
	*h = panelHeight;
}

void buttonCompile(w, h)
int w;
int h;
{
	/* Called from the init routine once we know the overall width 
	   of the window. Sets the sizes and offsets so that drawButton() 
	   will work.

	   'w' is the entire width of the window, including the 
	    button panel width. */

	BUTTON_DESC *b;
	int baseX, baseY;
	int row;
	int kernW, dkw, dsw, ns, deltaW, n;
	
	baseY = h;
	row = -1;
	for (b=buttonList; b; b=b->next)
	 { if (b->row != row)
	    { row = b->row;
	      baseY -= (buttonHeight + buttonSpacing);
	      baseX = w-(buttonSpacing+buttonWidth);
	      kernW = (b->rowMinW - (b->rowN-1)*buttonSpacing)/(b->rowSizeK);
	      deltaW = buttonWidth - b->rowMinW;
	      dkw = deltaW/b->rowSizeK;
	      n = 0;
	      if (b->rowN > 1)
	       { deltaW = deltaW%b->rowSizeK;
		 dsw = deltaW/(b->rowN-1);
		 ns = deltaW%(b->rowN-1);
	       }
	      else
	       { dsw = ns = 0;
	       }
	    }
	   b->w = b->sizeK*(kernW + dkw);
	   b->h = buttonHeight;
	   b->x = baseX;
	   b->y = baseY;
	   baseX += (buttonSpacing + dsw + b->w);
	   if (n++ < ns)
	    { baseX += 1;
	    }
	 }
}

void buttonResize(dw,dh)
int dw,dh;
{
	/* Called whenever we resize the window, to update the 
	   button offsets. */

	BUTTON_DESC *b;

	for (b=buttonList; b; b=b->next)
	 { b->x += dw;
	   b->y += dh;
	 }
}

drawAllButtons()
{
	BUTTON_DESC *b;

	for (b=buttonList; b; b=b->next)
	 { (*fxntab.drawButton)(b);
	 }
}

BUTTON_DESC *overButton(x, y)
int x, y;
{
	register BUTTON_DESC *b;

	if (x <= buttonSpacing ||
	    x >= panelWidth-buttonSpacing)
	 { return (0);
	 }
	for (b=buttonList; b; b=b->next)
	 { if (!(b->flags & BF_INACTIVE) &&
		(x >= b->x && x < b->x+b->w) && (y >= b->y && y < b->y+b->h))
	    { return (b);
	    }
	 }
	return (0);
}

onResizeEvent(win, w, h)
int win;
int w, h;
{
	winw = w;
	winh = h;

	if (hostType != GL_H)
	 { gdrResize (winw, winh);
	 }
	adjustScreenTrans (winw, winh);
}

onRedrawEvent(win)
int win;
{
	if (win == MAINWIN)
	 { if (redrawFxn)
	    { (*redrawFxn)(redrawArg);
	    }
	 }
	else if (win == CTRLWIN)
	 { drawAllButtons();
	 }
}

static int dragX0;
static int dragY0;
static int dragging;
static BUTTON_DESC *buttonPressed;
static BUTTON_DESC *buttonInFocus;
static int mouseButtonDown;

#define BUTTON1	1
#define BUTTON2	2
#define BUTTON3	3

void onButtonEvent(win, num, pressed, x, y)
int win;
int num;
int pressed;
int x, y;
{
	BUTTON_DESC *bd;

	if (pressed)
	 { if (mouseButtonDown)
	    { return;
	    }
	   if (win == MAINWIN)
	    { if (num == BUTTON1 || num == BUTTON2)
	       { dragX0 = x;
		 dragY0 = y;
		 dragging = 1;
	       }
	    }
	   else if (win == CTRLWIN)
	    { if (num == BUTTON1)
	       { bd = overButton(x, y);
		 if (bd && bd->fxn)
		  { (*bd->fxn)(bd->num, BF_PRESSED, bd->arg);
		  }
		 buttonPressed = bd;
	       }
	    }
	   mouseButtonDown = num;
	 }
	else
	 { if (mouseButtonDown != num)
	    { return;
	    }
	   if (dragging > 0)
	    { dragging = 0;
	    }
	   else if (dragging == 1)
	    { /* onSelect (dragX0, dragY0); */
	      dragging = 0;
	    }
	   else if (bd = buttonPressed)
	    { if (bd->fxn)
	       { (*bd->fxn)(bd->num, BF_RELEASED, bd->arg);
	       }
	      buttonPressed = 0;
	    }
	   mouseButtonDown = 0;
	 }
}

void onMotionEvent (win, x, y)
int win, x, y;
{
	BUTTON_DESC *bd;

	if (dragging && viewCtrlMode)
	 { onDrag (mouseButtonDown, dragX0, dragY0, x, y);
	   dragX0 = x;
	   dragY0 = y;
	   dragging++;
	 }
	bd = NULL;
	if (win == CTRLWIN)
	 { bd = overButton (x, y);
	 }
	if (bd != buttonInFocus)
	 { if (buttonInFocus && buttonInFocus->fxn)
	    { (*buttonInFocus->fxn)(buttonInFocus->num, BF_OUTFOCUS,
				    buttonInFocus->arg);
	    }
	   if (bd && bd->fxn)
	    { (*bd->fxn)(bd->num, BF_INFOCUS, bd->arg);
	    }
	   buttonInFocus = bd;	     
	 }
}

void onFocusChange (win)
int win;
{
	if (win != MAINWIN)
	 { if (dragging)
	    { dragging = 0;
	    }
	 }
	if (win != CTRLWIN)
	 { if (buttonInFocus && buttonInFocus->fxn)
	    { (*buttonInFocus->fxn)(buttonInFocus->num, BF_OUTFOCUS,
				    buttonInFocus->arg);
	      buttonInFocus = NULL;
	    }
	 }
	if (mouseButtonDown)
	 { mouseButtonDown = 0;
	 }
	if (win == CTRLWIN)
	 { BUTTON_DESC *bd;
	   Grf4Event evx;

	   (*fxntab.pollEvent)(win, GRF_CURSOR, &evx);
	   bd = overButton (evx.val1, evx.val2);
	   if (bd && bd->fxn)
	    { (*bd->fxn)(bd->num, BF_INFOCUS, bd->arg);
	      buttonInFocus = bd;	     
	    }
	 }
}

static void onButton(n, code, mode)
int n, code;
long mode;
{
	if (code == BF_PRESSED)
	 { if (viewCtrlMode != mode)
	    { switch (viewCtrlMode)
	       { case YAW_PITCH_CTRL:
		  { clearButton (yawCtrlButton);
		    clearButton (pitchCtrlButton);
		    break;
		  }
		 case X_Y_CTRL:
		  { clearButton (xCtrlButton);
		    clearButton (yCtrlButton);
		    break;
		  }
		 case PANY_PANX_CTRL:
		  { clearButton (panXCtrlButton);
		    clearButton (panYCtrlButton);
		    break;
		  }
		 case ZOOM_CTRL:
		  { clearButton (zoomCtrlButton);
		    break;
		  }
		 case TILT_CTRL:
		  { clearButton (tiltCtrlButton);
		    break;
		  }
	       }
	      switch (mode)
	       { case YAW_PITCH_CTRL:
		  { lightButton (yawCtrlButton);
		    lightButton (pitchCtrlButton);
		    break;
		  }
		 case X_Y_CTRL:
		  { lightButton (xCtrlButton);
		    lightButton (yCtrlButton);
		    break;
		  }
		 case PANY_PANX_CTRL:
		  { lightButton (panXCtrlButton);
		    lightButton (panYCtrlButton);
		    break;
		  }
		 case ZOOM_CTRL:
		  { lightButton (zoomCtrlButton);
		    break;
		  }
		 case TILT_CTRL:
		  { lightButton (tiltCtrlButton);
		    break;
		  }
	       }
	      viewCtrlMode = mode;
	      xMotionEnable = yMotionEnable = 1;
	    }
	   else
	    { int butx, buty;

	      switch (viewCtrlMode)
	       { case YAW_PITCH_CTRL:
		  { butx = yawCtrlButton;
		    buty = pitchCtrlButton;
		    break;
		  }
		 case X_Y_CTRL:
		  { butx = xCtrlButton;
		    buty = yCtrlButton;
		    break;
		  }
		 case PANY_PANX_CTRL:
		  { butx = panXCtrlButton;
		    buty = panYCtrlButton;
		    break;
		  }
		 default:
		  { return;
		  }
	       }
	      if (n == butx)
	       { if (xMotionEnable)
		  { clearButton(butx);
		    xMotionEnable = 0;
		  }
		 else
		  { lightButton(butx);
		    xMotionEnable = 1;
		  }
	       }
	      else if (n == buty)
	       { if (yMotionEnable)
		  { clearButton(buty);
		    yMotionEnable = 0;
		  }
		 else
		  { lightButton(buty);
		    yMotionEnable = 1;
		  }
	       }
	      else
	       { grf4Fatal ("onButton(): bogus button no. %d\n", n);
	       }
	    }
	 }
}


static bumpRpyValue (idx, inc)
int idx;
int inc;
{
	struct rpyButton *bd;
	char buf[32];
	TRSF Enew;
	int n;

	bd = &rpyButtons[idx];

	Enew = E;
	n = (NINT(bd->val)+inc)/inc;

	bd->val = range180 ((double)n*inc);
	rpyToTrsf (&Enew, rpyButtons[ROLL_BUTTON].val, 
			  rpyButtons[PITCH_BUTTON].val, 
			  rpyButtons[YAW_BUTTON].val);
	switch (idx)
	 { case ROLL_BUTTON:
	    { sprintf (buf, "roll:  %6.1f", bd->val);
	      break;
	    }
	   case PITCH_BUTTON:
	    { sprintf (buf, "pitch: %6.1f", bd->val);
	      break;
	    }
	   case YAW_BUTTON:
	    { sprintf (buf, "yaw:   %6.1f", bd->val);
	      break;
	    }
	 }
	grf4SetButtonName (bd->dispButton, buf);
	doSetView (&Enew, /*updateButtons=*/NO);
	if (redrawFxn)
	 { (*redrawFxn)(redrawArg);
         }
}

static void onRpyButton(n, code, idx)
int n, code;
long idx;
{
	struct rpyButton *bd;

	bd = &rpyButtons[idx];

	if (code == BF_PRESSED)
	 { if (n == bd->upButton)
	    { bumpRpyValue (idx, 90);
	    }
	   else if (n == bd->downButton)
	    { bumpRpyValue (idx, -90);
	    }
	   else
	    { grf4Fatal ("onRpyButton(): bogus button: n=%d idx=%d\n",n,idx);
	    }			  
	 }
}

clearButton(num)
int num;
{
	int flags;
	
	flags = grf4GetButtonFlags (num);
	grf4SetButtonFlags (num, flags & ~(BF_ILLUMINATED));
}

lightButton(num)
int num;
{
	int flags;
	
	flags = grf4GetButtonFlags (num);
	grf4SetButtonFlags (num, flags | BF_ILLUMINATED);
}

onDrag(button, x1, y1, x2, y2)
int button;
int x1, y1, x2, y2;
{
	TRSF D, Enew;
	float roll, pitch, yaw;
	float px, py, pz;
	int deltaX, deltaY;
	float scale;

	deltaX = x2-x1;
	deltaY = y2-y1;
	
	if (viewCtrlMode == NO_VIEW_CTRL)
	 { return;
	 }

	if (button == BUTTON1)
	 { scale = 1.0;
	 }
	else
	 { scale = 0.1;
	 }

#define DEADBAND	5

	switch (viewCtrlMode)
	 { case YAW_PITCH_CTRL:
	    { 
	      yaw = pitch = 0.0;
	      deadBandClip (&deltaX, &deltaY, DEADBAND);
	      if (yMotionEnable)
	       { yaw = -scale*90.0*deltaY/winh;
	       }
	      if (xMotionEnable)
	       { pitch = scale*90.0*deltaX/winw;
	       }
	      zeroVect (&D.p);
	      rpyToTrsf (&D, 0.0, pitch, yaw);
	      Enew = E;
	      zeroVect (&Enew.p);
	      multTrsf (&Enew, &D, &Enew);
	      Enew.p = E.p;
	      break;
	    }
	   case PANY_PANX_CTRL:
	    { 
	      yaw = pitch = 0.0;
	      deadBandClip (&deltaX, &deltaY, DEADBAND);
	      if (yMotionEnable)
	       { yaw = scale*90.0*deltaY/winh;
	       }
	      if (xMotionEnable)
	       { pitch = -scale*90.0*deltaX/winw;
	       }
	      zeroVect (&D.p);
	      rpyToTrsf (&D, 0.0, pitch, yaw);
	      Enew = E;
	      multTrsf (&Enew, &D, &Enew);
	      break;
	    }
	   case TILT_CTRL:
	    { 
	      roll = scale*90.0*deltaX/winw;
	      zeroVect (&D.p);
	      rpyToTrsf (&D, roll, 0.0, 0.0);
	      Enew = E;
	      multTrsf (&Enew, &D, &Enew);
	      break;
	    }
	   case X_Y_CTRL:
	    { if (perspective)
	       { scale *= (desiredNear+desiredFar)/(2*desiredNear);
	       }
	      else
	       { scale = 1.0;
	       }
	      Enew = E;
	      if (xMotionEnable)
	       { Enew.p.x += scale*virtW*deltaX/winw;
	       }
	      if (yMotionEnable)
	       { Enew.p.y += scale*virtH*deltaY/winh;
	       }
	      break;
	    }
	   case ZOOM_CTRL:
	    { float r1, r2;

	      scale *= (desiredFar-desiredNear)/(2*virtW);
	      Enew = E;
	      x1 -= winw/2;
	      x2 -= winw/2;
	      y1 -= winh/2;
	      y2 -= winh/2;
	      r1 = sqrt((double)(x1*x1+y1*y1));
	      r2 = sqrt((double)(x2*x2+y2*y2));
	      Enew.p.z += scale*(r2-r1)*virtW/winw;
	    }
	 }
	doSetView (&Enew, /*updateButtons=*/YES);
	if (redrawFxn)
	 { (*redrawFxn)(redrawArg);
         }
}

static void updateRpyButtons(r, p, y)
double r, p, y;
{
	char buf[64];
	struct rpyButton *bd;

	bd = &rpyButtons[ROLL_BUTTON];
	if (r != bd->val)
	 { sprintf (buf, "roll:  %6.1f", r);
	   grf4SetButtonName (bd->dispButton, buf);
	   bd->val = r;
	 }

	bd = &rpyButtons[PITCH_BUTTON];
	if (p != bd->val)
	 { sprintf (buf, "pitch: %6.1f", p);
	   grf4SetButtonName (bd->dispButton, buf);
	   bd->val = p;
	 }

	bd = &rpyButtons[YAW_BUTTON];
	if (y != bd->val)
	 { sprintf (buf, "yaw:   %6.1f", y);
	   grf4SetButtonName (bd->dispButton, buf);
	   bd->val = y;
	 }
}

void *grf4DefMaterialD(matd)
MATERIAL_DEF *matd;
{
	return (*fxntab.defMaterialD)(matd);
}

void *grf4DefMaterial(diffuse, ambient, emission, specular, shine, alpha)
unsigned long diffuse, ambient, emission, specular;
double shine, alpha;
{
	/* used to define GRF4 materials. The format and arguments
	   are virtually identical to those used in an old GL
	   material definition, except that RGB values are expressed
	   using bitmasks packed into the lower 24 bits of a long word.
	 */
	return (*fxntab.defMaterial)(diffuse, ambient, emission,
				     specular, shine, alpha);
}

void grf4SetMaterial(m)
void *m;
{
	currentMaterial = m;
	(*fxntab.setMaterial)(m);
}

void grf4PushMaterial()
{
	pushMaterial();
}

void grf4PopMaterial()
{
	popMaterial();
}

void *grf4DefLight(color, ambient, x,y,z,w)
unsigned long color, ambient;
double x,y,z,w;
{
	return (*fxntab.defLight)(color, ambient, x,y,z,w);
}

void grf4SetLight(n,l)
int n;
void *l;
{
	(*fxntab.loadTrans)(&E);
	(*fxntab.setLight)(n,l);
	(*fxntab.loadTrans)(&T);
}


deadBandClip (xp, yp, band)
int *xp;
int *yp;
int band;
{
}

int alistCmp(a1, a2)
AlistElem *a1, *a2;
{
	if (a1->left < a2->left)
	 { return (-1);
	 }
	else if (a1->left > a2->left)
	 { return (1);
	 }
	else
	 { return (0);
	 }
}

void alistBuild (fwdp, revp, base, n)
AlistElem **fwdp, **revp;
AlistElem *base;
int n;
{
	AlistElem *fwd, *rev;
	int i;

	fwd = (AlistElem*)malloc(n*sizeof(AlistElem));
	for (i=0; i<n; i++)
	 { fwd[i] = base[i];
	 }
	qsort (fwd, n, sizeof(AlistElem), alistCmp);

	rev = (AlistElem*)malloc(n*sizeof(AlistElem));
	for (i=0; i<n; i++)
	 { rev[i].right = base[i].left;
	   rev[i].left = base[i].right;
	 }
	qsort (rev, n, sizeof(AlistElem), alistCmp);

	*fwdp = fwd;
	*revp = rev;
}

int alistSearch (rp, key, base, n)
int *rp;
int key;
AlistElem *base;
int n;
{
	AlistElem keyElem;
	AlistElem *ptr;

	keyElem.left = key;
	ptr = (AlistElem*)bsearch(&keyElem,base,n,sizeof(AlistElem),alistCmp);
	if (ptr)
	 { *rp = ptr->right;
	   return (1);
	 }
	else
	 { return (0);
	 }	
}

void grf4Fatal VA_ARGDEF((char* fmt, ...))
{
	va_list ap;
#if !USE_STDARG
	char *fmt;

	va_start(ap);
	fmt = va_arg(ap, char *);
#else
	va_start(ap,fmt);
#endif
	fprintf (stderr, "ERROR -- ");
	vfprintf (stderr, fmt, ap);
	va_end (ap);
	exit (1);
}

void grf4CalcVirtualCursor (vxp, vyp, x, y)
double *vxp, *vyp;
int x, y;
{
	*vxp = virtL + x*(virtW)/(double)winw;
	*vyp = virtB + y*(virtH)/(double)winh;
}
