/*-=-=-=-=-=-=-=-=-=-=-=- RCCL/RCI Notice of Copyright -=-=-=-=-=-=-=-=-=-=-*
 *									    *
 *	Copyright (C) 1991 by John E. Lloyd.				    *
 *									    *
 *	Information on copyright permissions, as well as a complete	    *
 *	disclaimer of warranty, is given in the file COPYRIGHT.		    *
 *									    *
 *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

/* CRSplus A460 stuff */

#include <stdio.h>
#include <math.h>

#include "../h/cdefs.h"
#include "../h/rccl.h"	/* XXX MUST include rccl.h before grf4.h */
#include "../h/puma_kynvar.h"

#include "../h/grf4.h"

#define NUM_CIRC_EDGES		12

#define J1			0
#define J2			1
#define J3			2
#define J4			3
#define J5			4
#define J6			5

/* Kinematic dimensions are specified in millimeters so do not convert. */

#define A2			(gen->dh[J2].a)
#define D3			(gen->dh[J3].d)
#define A3			(gen->dh[J3].a)
#define D4			(gen->dh[J4].d)

#define BEIGE			RGBPACK(225, 160,  90)
#define DARK_BROWN		RGBPACK( 85,  55,  35)
#define DARK_GREY		RGBPACK( 50,  50,  50)
#define BLACK			RGBPACK(  0,   0,   0)
#define WHITE			RGBPACK(255, 255, 255)
#define SILVER			RGBPACK(200, 200, 220)
#define GOLD			RGBPACK(220, 200, 220)
#define AMBER			RGBPACK(255, 125,   0)

#define R2			1.414213562

/* The following graphic dimensions are in millimeters. */

#define C460_WAIST_RADIUS0	10.50
#define C460_WAIST_RADIUS1	9.50
#define C460_WAIST_HEIGHT0	13.00
#define C460_WAIST_HEIGHT1	6.50

#define C460_BASE_SIZE	        (C460_WAIST_RADIUS0 * 2.0)
#define C460_BASE_HEIGHT	3.00

#define C460_YOKE_RADIUS	6.75
#define C460_YOKE_LENGTH	11.00
#define C460_YOKE_DEPTH		3.00
#define C460_YOKE_BEVEL		(C460_YOKE_RADIUS + 1.60)

#define C460_SHOULDER_RADIUS0	(C460_YOKE_RADIUS)
#define C460_SHOULDER_RADIUS1	3.75
#define C460_SHOULDER_DEPTH	16.50
	
#define C460_ELBOW_RADIUS	4.75
#define C460_ELBOW_DEPTH	14.00

#define C460_FORE_RADIUS	5.00
#define C460_FORE_DEPTH		4.00

#define C460_UPPER_LENGTH	30.480	/* should be A2 */
#define C460_UPPER_DEPTH	10.00
#define C460_UPPER_RADIUS0	6.75
#define C460_UPPER_RADIUS1	5.50

#define C460_LOWER_LENGTH	30.480	/* should be D2 */

#define C460_LOWER1_LENGTH	9.50
#define C460_LOWER1_WIDTH	8.00
#define C460_LOWER1_DEPTH	8.00
#define C460_LOWER1_RADIUS	5.50

#define C460_LOWER2_LENGTH0	16.98
#define C460_LOWER2_LENGTH1	15.00
#define C460_LOWER2_LENGTH2	9.00
#define C460_LOWER2_WIDTH0	8.00
#define C460_LOWER2_WIDTH1	8.00
#define C460_LOWER2_WIDTH2	10.00
#define C460_LOWER2_DEPTH	8.00
#define C460_LOWER2_RADIUS	4.75

#define C460_WRIST1_RADIUS	4.00
#define C460_WRIST1_DEPTH	5.50

#define C460_WRIST2_RADIUS	2.50
#define C460_WRIST2_DEPTH	6.00

#define C460_WRIST3_RADIUS	3.00
#define C460_WRIST3_DEPTH	2.00

#define C460_HAND_STEM_LENGTH	7.10
#define C460_HAND_STEM_WIDTH	5.20
#define C460_HAND_SLIDE_LENGTH	9.00
#define C460_HAND_SLIDE_WIDTH	1.80
#define C460_HAND_FINGER_LENGTH	3.00
#define C460_HAND_FINGER_WIDTH	1.50
#define C460_HAND_DEPTH		3.70
#define C460_HAND_SIZE		10.10

static float c460_yoke_bot[] =
  {
    0.0,			C460_YOKE_LENGTH + C460_YOKE_RADIUS,
    -C460_YOKE_RADIUS/R2,	C460_YOKE_LENGTH + C460_YOKE_RADIUS/R2,
    -C460_YOKE_RADIUS,		C460_YOKE_LENGTH,
    -C460_YOKE_RADIUS,		0.0,
    C460_YOKE_RADIUS,		0.0,
    C460_YOKE_RADIUS,		C460_YOKE_LENGTH,
    C460_YOKE_RADIUS/R2,	C460_YOKE_LENGTH + C460_YOKE_RADIUS/R2,
  };

static float c460_yoke_top[] =
  {
    0.0,			C460_YOKE_LENGTH + C460_YOKE_RADIUS,
    -C460_YOKE_RADIUS/R2,	C460_YOKE_LENGTH + C460_YOKE_RADIUS/R2,
    -C460_YOKE_RADIUS,		C460_YOKE_LENGTH,
    -C460_YOKE_BEVEL,		0.0,
    C460_YOKE_BEVEL,		0.0,
    C460_YOKE_RADIUS,		C460_YOKE_LENGTH,
    C460_YOKE_RADIUS/R2,	C460_YOKE_LENGTH + C460_YOKE_RADIUS/R2,
  };

static char* c460_yoke_hidden = "1111111";

static float c460_upper_prism[] =
  {
    0.0,			C460_UPPER_LENGTH + C460_UPPER_RADIUS1,
    -C460_UPPER_RADIUS1/R2,	C460_UPPER_LENGTH + C460_UPPER_RADIUS1/R2,
    -C460_UPPER_RADIUS1,	C460_UPPER_LENGTH,
    -C460_UPPER_RADIUS0,	0.0,
    -C460_UPPER_RADIUS0/R2,	-C460_UPPER_RADIUS0/R2,
    0.0,			-C460_UPPER_RADIUS0,
    C460_UPPER_RADIUS0/R2,	-C460_UPPER_RADIUS0/R2,
    C460_UPPER_RADIUS0,		0.0,
    C460_UPPER_RADIUS1,		C460_UPPER_LENGTH,
    C460_UPPER_RADIUS1/R2,	C460_UPPER_LENGTH + C460_UPPER_RADIUS1/R2,
  };

static char* c460_upper_hidden = "1111111111";

static float c460_lower1_prism[] =
  {
    -C460_LOWER1_WIDTH / 2.0,	C460_LOWER1_LENGTH,
    -C460_LOWER1_RADIUS,	0.0,
    -C460_LOWER1_RADIUS/R2,	-C460_LOWER1_RADIUS/R2,
    0.0,			-C460_LOWER1_RADIUS,
    C460_LOWER1_RADIUS/R2,	-C460_LOWER1_RADIUS/R2,
    C460_LOWER1_RADIUS,		0.0,
    C460_LOWER1_WIDTH / 2.0,	C460_LOWER1_LENGTH,
  };

static char* c460_lower1_hidden = "1111111";

static float c460_lower2_prism[] =
  {
    -C460_LOWER2_WIDTH0 / 2.0,	C460_LOWER2_LENGTH0,
    -C460_LOWER2_WIDTH1 / 2.0,	C460_LOWER2_LENGTH1,
    -C460_LOWER2_WIDTH2 / 2.0,	C460_LOWER2_LENGTH2,
    -C460_LOWER2_RADIUS,	0.0,
    -C460_LOWER2_RADIUS/R2,	-C460_LOWER2_RADIUS/R2,
    0.0,			-C460_LOWER2_RADIUS,
    C460_LOWER2_RADIUS/R2,	-C460_LOWER2_RADIUS/R2,
    C460_LOWER2_RADIUS,		0.0,
    C460_LOWER2_WIDTH2 / 2.0,	C460_LOWER2_LENGTH2,
    C460_LOWER2_WIDTH1 / 2.0,	C460_LOWER2_LENGTH1,
    C460_LOWER2_WIDTH0 / 2.0,	C460_LOWER2_LENGTH0,
  };

static char* c460_lower2_hidden = "11111111111";

static int capacity;

Grf4Object
  *c460base,
  *c460waist1,
  *c460waist2,
  *c460yoke1,
  *c460yoke2,
  *c460shoulder1,
  *c460shoulder2,
  *c460elbow,
  *c460fore,
  *c460upper,
  *c460lower,
  *c460lower1,
  *c460lower2,
  *c460wrist1,
  *c460wrist2,
  *c460wrist3;

drawCRSplusHand(open)
  int open;
  {
    float m, h;

    m = C460_HAND_SLIDE_LENGTH;
    h = (open ? m / 2.0 : m / 4.0);

    grf4Color(SILVER);

    /* draw the stem and the slide */

    grf4Move(0.0, 0.0, 0.0);
    grf4Draw(C460_HAND_STEM_LENGTH, 0.0, 0.0);
    grf4Move(C460_HAND_STEM_LENGTH, 0.0 - C460_HAND_SLIDE_LENGTH / 2.0, 0.0);
    grf4Draw(C460_HAND_STEM_LENGTH, 0.0 + C460_HAND_SLIDE_LENGTH / 2.0, 0.0);

    /* draw the two fingers */

    grf4Move(C460_HAND_SIZE,		0.0 - h, 0.0);
    grf4Draw(C460_HAND_STEM_LENGTH,	0.0 - h, 0.0);
    grf4Move(C460_HAND_SIZE,		0.0 + h, 0.0);
    grf4Draw(C460_HAND_STEM_LENGTH,	0.0 + h, 0.0);
  }

#define ZSEP	1.0
#define ZLEN	10.0

drawFrame ()
{
	grf4Move(-ZSEP, 0.0, 0.0);
    	grf4Draw(-ZSEP, 0.0, ZLEN);
	grf4Move( ZSEP, 0.0, 0.0);
    	grf4Draw( ZSEP, 0.0, ZLEN);
	grf4Move( 0.0, 0.0, 0.0);
    	grf4Draw( ZLEN, 0.0, 0.0);
}


defineC460Links()
  {
    void *baseMat;
    void *bodyMat;
    void *wristMat;

    baseMat	= grf4DefMaterial(DARK_GREY, DARK_GREY,
		    0, 0x000000, 0.0, 1.0);
    bodyMat	= grf4DefMaterial(SILVER, SILVER,
		    0, 0x000000, 0.0, 1.0);
    wristMat	= grf4DefMaterial(DARK_GREY, DARK_GREY,
		    0, 0x808080, 50.0, 1.0);

    c460base		= grf4GenBox(
			    C460_BASE_SIZE,
			    C460_BASE_SIZE,
			    C460_BASE_HEIGHT,
			    baseMat);

    c460waist1		= grf4GenCone(NUM_CIRC_EDGES,
			    C460_WAIST_RADIUS0,
			    C460_WAIST_RADIUS1,
			    C460_WAIST_HEIGHT0,
			    baseMat);

    c460waist2		= grf4GenDisc(NUM_CIRC_EDGES,
			    C460_WAIST_RADIUS1,
			    C460_WAIST_HEIGHT1,
			    baseMat);

    c460yoke1		= grf4GenPrism(ARRAY_SIZE(c460_yoke_bot) / 2,
			    c460_yoke_bot,
			    c460_yoke_top,
			    c460_yoke_hidden,
			    C460_YOKE_DEPTH,
			    baseMat);

    c460yoke2		= grf4GenPrism(ARRAY_SIZE(c460_yoke_top) / 2,
			    c460_yoke_top,
			    c460_yoke_bot,
			    c460_yoke_hidden,
			    C460_YOKE_DEPTH,
			    baseMat);

    c460shoulder1	= grf4GenCone(NUM_CIRC_EDGES,
			    C460_SHOULDER_RADIUS0,
			    C460_SHOULDER_RADIUS1,
			    C460_SHOULDER_DEPTH,
			    bodyMat);

    c460shoulder2	= grf4GenCone(NUM_CIRC_EDGES,
			    C460_SHOULDER_RADIUS1,
			    C460_SHOULDER_RADIUS0,
			    C460_SHOULDER_DEPTH,
			    bodyMat);

    c460elbow		= grf4GenDisc(NUM_CIRC_EDGES,
			    C460_ELBOW_RADIUS,
			    C460_ELBOW_DEPTH,
			    baseMat);

    c460fore		= grf4GenDisc(NUM_CIRC_EDGES,
			    C460_FORE_RADIUS,
			    C460_FORE_DEPTH,
			    bodyMat);

    c460upper		= grf4GenPrism(ARRAY_SIZE(c460_upper_prism) / 2,
			    c460_upper_prism,
			    (float *) 0,
			    c460_upper_hidden,
			    C460_UPPER_DEPTH,
			    bodyMat);

    c460lower1		= grf4GenPrism(ARRAY_SIZE(c460_lower1_prism) / 2,
			    c460_lower1_prism,
			    (float *) 0,
			    c460_lower1_hidden,
			    C460_LOWER1_DEPTH,
			    bodyMat);

    c460lower2		= grf4GenPrism(ARRAY_SIZE(c460_lower2_prism) / 2,
			    c460_lower2_prism,
			    (float *) 0,
			    c460_lower2_hidden,
			    C460_LOWER2_DEPTH,
			    bodyMat);

    c460wrist1		= grf4GenDisc(NUM_CIRC_EDGES,
			    C460_WRIST1_RADIUS,
			    C460_WRIST1_DEPTH,
			    wristMat);

    c460wrist2		= grf4GenDisc(NUM_CIRC_EDGES,
			    C460_WRIST2_RADIUS,
			    C460_WRIST2_DEPTH,
			    wristMat);

    c460wrist3		= grf4GenDisc(NUM_CIRC_EDGES,
			    C460_WRIST3_RADIUS,
			    C460_WRIST3_DEPTH,
			    wristMat);

    capacity = grf4Capacity();
  }

extern int special;


#define XX    10
#define Y0    44.54773
#define Z0    2.12132
#define Y1    63.63961
#define Z1    49.49747
#define Y2    49.49747
#define Z2    63.63961
#define Y3    2.12132
#define Z3    44.54773

static VECT crvP[4] =
 {  { XX, Y0, Z0 },
    { XX, Y1, Z1 },
    { XX, Y2, Z2 },
    { XX, Y3, Z3 }
 };
#define NSAMP	30

drawSpecialCurve()
{
	double a3x, a2x, a1x, a0x;
	double a3y, a2y, a1y, a0y;
	double a3z, a2z, a1z, a0z;

	VECT p;
	double s;
	int k;

	a3x = crvP[3].x - 3*crvP[2].x + 3*crvP[1].x -   crvP[0].x;
	a2x =          3*crvP[2].x - 6*crvP[1].x + 3*crvP[0].x;
	a1x =                     3*crvP[1].x - 3*crvP[0].x;
	a0x =                                  crvP[0].x;

	a3y = crvP[3].y - 3*crvP[2].y + 3*crvP[1].y -   crvP[0].y;
	a2y =          3*crvP[2].y - 6*crvP[1].y + 3*crvP[0].y;
	a1y =                     3*crvP[1].y - 3*crvP[0].y;
	a0y =                                  crvP[0].y;

	a3z = crvP[3].z - 3*crvP[2].z + 3*crvP[1].z -   crvP[0].z;
	a2z =          3*crvP[2].z - 6*crvP[1].z + 3*crvP[0].z;
	a1z =                     3*crvP[1].z - 3*crvP[0].z;
	a0z =                                  crvP[0].z;

	grf4Move (crvP[0].x, crvP[0].y, crvP[0].z);
	for (k=1; k<=NSAMP; k++)
	 { s = k/(double)NSAMP;

	   p.x = ((a3x*s + a2x)*s + a1x)*s + a0x;
	   p.y = ((a3y*s + a2y)*s + a1y)*s + a0y;
	   p.z = ((a3z*s + a2z)*s + a1z)*s + a0z;
	   grf4Draw (p.x, p.y, p.z);
	 }
}


drawC460(T, kyn, var, drawHand, handpos)
  register TRSF *T;
  KYN *kyn;
  void *var;
  int drawHand;
  int handpos;



  /* Clear the display and draw a robot, given a set of joint angles.
     At our site, the CRSplus A460 is inverted and suspended from a large
     metal frame, so the simulation is inverted also.
  */
  {
    PUMA_VAR *pvar = (PUMA_VAR *) var;
    GEN_KYN *gen = kyn->gen;

    TRSF trans, t;

    /* Reference frame. */

    grf4GetTrans(&trans);
/*  multTrsfRpy(&trans, 0.0, 0.0, 180.0); */

    /* Base. */

    t = trans;
    multTrsfXyz(&t,
      -C460_BASE_SIZE / 2.0,
      -C460_BASE_SIZE / 2.0,
      0.0);
    grf4SetTrans(&t);
    grf4DrawObject(c460base);

    multTrsfXyz(&trans,
      0.0,
      0.0,
      C460_BASE_HEIGHT);
    grf4SetTrans(&trans);
    grf4DrawObject(c460waist1);

    /* T[0] */

    multTrsfXyz(&trans,
      0.0,
      0.0,
      C460_WAIST_HEIGHT0 + C460_WAIST_HEIGHT1 + C460_YOKE_LENGTH);
    T[0] = trans;
    grf4SetTrans(&trans);

    if (special == 3)
     { grf4Color (GRF_YELLOW);
       grf4Move (0.001, 20.0, 65.5);
       grf4Draw (0.001, -20.0, 65.5);
     }
    else if (special == 2)
     { grf4Color (GRF_YELLOW);
       grf4Move (XX, 30, 30);
       grf4Draw (XX, 43.1, 43.1);
     }
    else if (special == 1)
     { grf4Color (GRF_YELLOW);
       drawSpecialCurve();
     }

    /* T[1] */

    pumaMultA1(&T[1], &T[0], kyn, pvar);

    /* Waist. */

    trans = T[1];

    multTrsfRpy(&trans, 0.0, 90.0, 90.0);
    multTrsfXyz(&trans,
      0.0,
      0.0,
      0.0 - C460_WAIST_HEIGHT1 - C460_YOKE_LENGTH);
    grf4SetTrans(&trans);
    grf4DrawObject(c460waist2);

    /* Yoke. */

    t = trans;
    multTrsfRpy(&t, 90.0, 0.0, 90.0);
    multTrsfXyz(&t,
      0.0,
      C460_WAIST_HEIGHT1,
      0.0 - (C460_YOKE_DEPTH + C460_UPPER_DEPTH) / 2.0);
    grf4SetTrans(&t);
    grf4DrawObject(c460yoke1);

    t = trans;
    multTrsfRpy(&t, 90.0, 0.0, 90.0);
    multTrsfXyz(&t,
      0.0,
      C460_WAIST_HEIGHT1,
      0.0 - (C460_YOKE_DEPTH - C460_UPPER_DEPTH) / 2.0);
    grf4SetTrans(&t);
    grf4DrawObject(c460yoke2);

    /* Shoulder. */

    trans = T[1];
    multTrsfXyz(&trans,
      0.0,
      0.0,
      C460_UPPER_DEPTH / 2.0);
    grf4SetTrans(&trans);
    grf4DrawObject(c460shoulder1);

    trans = T[1];
    multTrsfXyz(&trans,
      0.0,
      0.0,
      0.0 - C460_SHOULDER_DEPTH - C460_UPPER_DEPTH / 2.0);
    grf4SetTrans(&trans);
    grf4DrawObject(c460shoulder2);

    /* T[2] */

    pumaMultA2(&T[2], &T[1], kyn, pvar);

    /* Upper arm. */
    /* Flip axis to point limb forwards. */

    trans = T[2];
    multTrsfXyz(&trans,
     -C460_UPPER_LENGTH,
      0.0,
      C460_UPPER_DEPTH / 2.0);
    multTrsfRpy(&trans, 90.0, 0.0, 180.0);
    grf4SetTrans(&trans);
    grf4DrawObject(c460upper);

    /* Elbow. */

    trans = T[2];
    multTrsfXyz(&trans,
      0.0,
      0.0,
      -C460_ELBOW_DEPTH / 2.0);
    grf4SetTrans(&trans);
    grf4DrawObject(c460elbow);

    /* T[3] and T[4] */

    pumaMultA3(&T[3], &T[2], kyn, pvar);
    pumaMultA4(&T[4], &T[3], kyn, pvar);

    /* Lower arm. */
#if 1
    trans = T[3];
    multTrsfXyz(&trans,
      0.0,
      C460_LOWER1_DEPTH / 2.0,
      -C460_LOWER_LENGTH);
    multTrsfRpy(&trans, 0.0, 0.0, 90.0);
    grf4SetTrans(&trans);
    grf4DrawObject(c460lower1);

    trans = T[3];
    multTrsfXyz(&trans,
      0.0,
      0.0,
      -C460_LOWER2_LENGTH0 - C460_FORE_DEPTH);
    grf4SetTrans(&trans);
    grf4DrawObject(c460fore);

    trans = T[4];
    multTrsfXyz(&trans,
      0.0,
      0.0,
      -C460_LOWER2_DEPTH / 2.0);
    grf4SetTrans(&trans);
    grf4DrawObject(c460lower2);
#endif
    /* T[5] and T[6] */

    pumaMultA5(&T[5], &T[4], kyn, pvar);
    pumaMultA6(&T[6], &T[5], kyn, pvar);
#if 1
    /* Wrist. */

    trans = T[4];
    multTrsfXyz(&trans,
      0.0,
      0.0,
      -C460_WRIST1_DEPTH / 2.0);
    grf4SetTrans(&trans);
    grf4DrawObject(c460wrist1);

    trans = T[5];
    grf4SetTrans(&trans);
    grf4DrawObject(c460wrist2);

    trans = T[6];
    multTrsfXyz(&trans,
      0.0,
      0.0,
      C460_WRIST2_DEPTH);
    grf4SetTrans(&trans);
    grf4DrawObject(c460wrist3);

    if (drawHand)
      {
	multTrsfRpy(&trans, 0.0, -90.0, -180.0);
	multTrsfXyz(&trans, C460_WRIST2_RADIUS, 0.0, 0.0);
	grf4SetTrans(&trans);
	drawCRSplusHand(handpos);
      }
#endif
  }

c460ViewSize(r, b, h, kyn)
  float *r, *b, *h;
  KYN *kyn;
  /* Return the dimensions of the view as radius, below, and height.
     At our site, the CRSplus A460 is inverted and suspended from a large
     metal frame, so the simulation is inverted also and the view offset
     compensate.
  */
  {
    GEN_KYN *gen = kyn->gen;

    *r = C460_UPPER_LENGTH
      + C460_LOWER_LENGTH
      + C460_WRIST2_DEPTH
      + C460_WRIST3_DEPTH
      + C460_HAND_SIZE;
    *b = C460_BASE_HEIGHT
      + C460_WAIST_HEIGHT0
      + C460_WAIST_HEIGHT1
      + C460_SHOULDER_RADIUS0;
    *b = -*b;
    *h = *r;
  }

c460LinkFrames(T, B, kyn, var)
  TRSF *T, *B;
  KYN *kyn;
  void *var;
  {
    PUMA_VAR *pvar = (PUMA_VAR *) var;

    T[0] = *B;

    multTrsfXyz(&T[0],
      0.0,
      0.0,
      C460_BASE_HEIGHT
        + C460_WAIST_HEIGHT0
        + C460_WAIST_HEIGHT1
        + C460_SHOULDER_RADIUS0);

    pumaMultA1(&T[1], &T[0], kyn, pvar);
    pumaMultA2(&T[2], &T[1], kyn, pvar);
    pumaMultA3(&T[3], &T[2], kyn, pvar);
    pumaMultA4(&T[4], &T[3], kyn, pvar);
    pumaMultA5(&T[5], &T[4], kyn, pvar);
    pumaMultA6(&T[6], &T[5], kyn, pvar);
  }
