#ifdef Tsp_TARGET

#include <stdio.h>
#include <cdefs.h>
#include <conc.h>
#include "tgen.h"
#include "servo.h"
#include <vector.h>
#include <puma_kynvar.h>

#define SEC_TO_TICKS	(1000000/64.)

extern int controlFxnCnt;
extern int controlFxnTime;
extern int robotComTime;
extern int forceComTime;

main()
{
	JntVec jc;
	TRSF TC, TH;
	Tgen *tg;
	int enc[8];

	printf ("Allocating trajectory generator ...\n");

	if ((tg = tgenAlloc ("caesar")) == NULL)
	 { printErrors ();
	   exit (1);
	 }
	printf ("done\n");

	tgenSetCartVel (tg, 200.0, -1);
	tgenSetCartAcc (tg, 1000.0, -1);

	printf ("Starting trajectory generator ...\n");
	if (tgenStart (tg, 0.01, LINK3OUT, LINK3IN, LINK2OUT, LINK2IN) < 0)
	 { printErrors();
	   exit (1);
	 }
	printf ("done\n");

	if (getRobotPosition (jc.v, "demohome", "caesar") < 0)
	 { printErrors();
	   exit (1);
	 }
	printJf ("demohome: %8.3f\n", jc.v, 6);
	angleToEncoder (enc, jc.v, tg->jls);
	printf  ("          %8d %8d %8d %8d %8d %8d\n", 
		 enc[0], enc[1], enc[2], enc[3], enc[4], enc[5]);
	printf ("moving to demohome (confirm):\n");
	while (getchar() != '\n')
	   ;
	tgenMoveJ (tg, &jc);

	do
	 { ProcWait (0.2*SEC_TO_TICKS);
	 }
	while (tgenIsBusy(tg));
	printf ("done\n");

	TH = tg->T6;
	printf ("Now doing simple box motion (confirm):\n");
	while (getchar() != '\n')
	   ;

	TC = TH;
	TC.p.x -= 50.0;
	TC.p.y -= 50.0;
	tgenSetPath (tg, TGEN_CART_PATH);
	tgenMoveC (tg, &TC, (TRSF*)0);
	tgenStop (tg, 1.0);
	TC.p.y += 100.0;
	tgenMoveC (tg, &TC, (TRSF*)0);
	tgenStop (tg, 1.0);
	TC.p.x += 100.0;
	tgenMoveC (tg, &TC, (TRSF*)0);
	tgenStop (tg, 1.0);
	TC.p.y -= 100.0;
	tgenMoveC (tg, &TC, (TRSF*)0);
	tgenStop (tg, 1.0);
	TC.p.x -= 100.0;
	tgenMoveC (tg, &TC, (TRSF*)0);
	tgenStop (tg, 1.0);

	do
	 { ProcWait (0.2*SEC_TO_TICKS);
	 }
	while (tgenIsBusy(tg));
	printf ("done\n");

	printf ("Continuous box motion (confirm):\n");
	while (getchar() != '\n')
	   ;

	TC = TH;
	TC.p.x -= 50.0;
	TC.p.y -= 50.0;
	tgenSetPath (tg, TGEN_CART_PATH);
	tgenMoveC (tg, &TC, (TRSF*)0);
	TC.p.y += 100.0;
	tgenMoveC (tg, &TC, (TRSF*)0);
	TC.p.x += 100.0;
	tgenMoveC (tg, &TC, (TRSF*)0);
	TC.p.y -= 100.0;
	tgenMoveC (tg, &TC, (TRSF*)0);
	TC.p.x -= 100.0;
	tgenMoveC (tg, &TC, (TRSF*)0);

	do
	 { ProcWait (0.2*SEC_TO_TICKS);
	 }
	while (tgenIsBusy(tg));

	printf ("Changing wrist configuration (confirm):\n");
	while (getchar() != '\n')
	   ;

	tgenMoveC (tg, &TH, (TRSF*)0);
	tgenStop (tg, 1.0);
	tgenSetPath (tg, TGEN_JNT_PATH);
	tgenSetConf (tg, PUMA_FLIP_CONF);
	tgenMoveC (tg, &TH, (TRSF*)0);
	tgenStop (tg, 1.0);
	tgenSetConf (tg, 0);
	tgenMoveC (tg, &TH, (TRSF*)0);
	tgenStop (tg, 1.0);

	do
	 { ProcWait (0.2*SEC_TO_TICKS);
	 }
	while (tgenIsBusy(tg));

	tgenRelease (tg);

	printf ("released\n");
	printf ("avg times (usec):\n");
	printf ("controlFxn %g  robotCom %g  forceCom %g\n",
		 controlFxnTime/(double)controlFxnCnt,
		 robotComTime/(double)controlFxnCnt,
		 forceComTime/(double)controlFxnCnt);
}

#endif /* Tsp_TARGET */
