/* Routines to help arrange proper byte order. They assume 4-byte ints 
   and 2-byte shorts. The byte ordering when packed is assumed to be
   little-endian (VAX-like; low-order byte has lowest address) */

#include "../h/packByte.h"

union _int {
  unsigned char c[4];
  int i;
};

union _short {
  unsigned char c[2];
  short s;
};

static int bigEndian = -1;

void packInt (int i, unsigned char *buf)
{
	union _int pack;

	if (bigEndian == -1)
	 { bigEndian = isBigEndian();
	 }
	pack.i = i;
	if (bigEndian)
	 { buf[0] = pack.c[3];
	   buf[1] = pack.c[2];
	   buf[2] = pack.c[1];
	   buf[3] = pack.c[0];
	 }
	else
	 { buf[0] = pack.c[0];
	   buf[1] = pack.c[1];
	   buf[2] = pack.c[2];
	   buf[3] = pack.c[3];
	 }
}

void pack3byteInt (int i, unsigned char *buf)
{
	union _int pack;

	if (bigEndian == -1)
	 { bigEndian = isBigEndian();
	 }
	pack.i = i;
	if (bigEndian)
	 { buf[0] = pack.c[3];
	   buf[1] = pack.c[2];
	   buf[2] = pack.c[1];
	 }
	else
	 { buf[0] = pack.c[0];
	   buf[1] = pack.c[1];
	   buf[2] = pack.c[2];
	 }
}

void packShort (short s, unsigned char *buf)
{
	union _short pack;

	if (bigEndian == -1)
	 { bigEndian = isBigEndian();
	 }
	pack.s = s;
	if (bigEndian)
	 { buf[0] = pack.c[1];
	   buf[1] = pack.c[0];
	 }
	else
	 { buf[0] = pack.c[0];
	   buf[1] = pack.c[1];
	 }
}

int unpackInt (unsigned char *buf)
{
	union _int pack;

	if (bigEndian == -1)
	 { bigEndian = isBigEndian();
	 }
	if (bigEndian)
	 { pack.c[3] = buf[0];
	   pack.c[2] = buf[1];
	   pack.c[1] = buf[2];
	   pack.c[0] = buf[3];
	 }
	else
	 { pack.c[0] = buf[0];
	   pack.c[1] = buf[1];
	   pack.c[2] = buf[2];
	   pack.c[3] = buf[3];
	 }
	return (pack.i);	
}

int unpack3byteInt (unsigned char *buf)
{
	union _int pack;

	if (bigEndian == -1)
	 { bigEndian = isBigEndian();
	 }
	if (bigEndian)
	 { pack.c[3] = buf[0];
	   pack.c[2] = buf[1];
	   pack.c[1] = buf[2];
	   pack.c[0] = (buf[2] & 0x80 ? 0xff : 0x0);
	 }
	else
	 { pack.c[0] = buf[0];
	   pack.c[1] = buf[1];
	   pack.c[2] = buf[2];
	   pack.c[3] = (buf[2] & 0x80 ? 0xff : 0x0);
	 }
	return (pack.i);	
}

short unpackShort (unsigned char *buf)
{
	union _short pack;

	if (bigEndian == -1)
	 { bigEndian = isBigEndian();
	 }
	if (bigEndian)
	 { pack.c[1] = buf[0];
	   pack.c[0] = buf[1];
	 }
	else
	 { pack.c[0] = buf[0];
	   pack.c[1] = buf[1];
	 }
	return (pack.s);	
}

int isBigEndian()
{
	union _short swap;
	swap.s = 1;
	return (swap.c[1] == 1);
}
