#include <sys/types.h>
#include <sys/file.h>
#include <sys/errno.h>
#include <sys/open.h>
#include <sys/cred.h>
#include <sys/uio.h>
#include <sys/conf.h>
#include <sys/ddi.h>
#include <sys/poll.h>
#include <sys/kmem.h>
#include <sys/cmn_err.h>
#include <sys/edt.h>
#include <sys/pio.h>
#include <sys/eisa.h>

#include "rciioctl.h"
#include "xydreg.h"

int xyddevflag = 0;
extern int xyd_internal_major;
extern XydSoftc xydi[];

#define IRQ_MASK	0x0010

void xydintr(unit)
int unit;
{
	/* Called in response to an interrupt from the XYD. The board can
	   generate 5 different types of interrupts, only one of which is
	   defined for our application. */

	XydSoftc *info;
	unsigned short pending;
	int temp;
	int s;

	info = &xydi[unit];

	pending = (info->xyd->cfg1stat & 
		   (XYD_DRDY1|XYD_DRDY2|XYD_DMATC1|XYD_DMATC2|XYD_CNTINT));
	if (pending & XYD_DRDY2)
	 { temp = info->xyd->portCD;	/* read port to clear interrupt */
	   if (info->flags & XYD_BUSY)
	    { s = splhi();
	      if (info->intrCnt < info->qsize)
	       { info->intrCnt++;		/* bump the counter */
		 pollwakeup (info->pollhead, POLLRDNORM);
	       }
	      splx(s);
	      pending &= ~(XYD_DRDY2);
	    }
	 }
	if (pending & (XYD_DRDY1|XYD_DMATC1|XYD_DMATC2|XYD_CNTINT))
	 { /* These shouldn't happen, but we check for them anyway, and
	      try to clear the interrupt in each case. */
	   if (pending & XYD_DRDY1)
	    { temp = info->xyd->portAB;
	    }
	   if (pending & XYD_DMATC1)
	    { info->xyd->dmaclr1 = 0;
	    }
	   if (pending & XYD_DMATC2)
	    { info->xyd->dmaclr2 = 0;
	    }
	   if (pending & XYD_CNTINT)
	    { info->xyd->cntintclr = 0;
	    }
	 }
	if (pending)
	 { /* declare a stray interrupt */
	   cmn_err (CE_CONT, "xyd%d: stray interrupt(s); mask=0x%x\n",
			      unit, pending);
	 }
}

xydedtinit(e)
edt_t *e;
{
	/* Initialization routine. Verify that the board is there, and
	   initialize its registers to something reasonable. */

	int i, vec;
	volatile Xyd *xyd;
	piomap_t *pmap;

	pmap = pio_mapalloc(e->e_bus_type, e->e_adap, &e->e_space[0],
			    PIOMAP_FIXED, "xyd");
	if (pmap == 0)
	 { return;
	 }
	xyd = (Xyd*)pio_mapaddr (pmap, e->e_space[0].ios_iopaddr);
	if (xyd == 0)
	 { cmn_err (CE_WARN, "xyd: unit %d, pio_mapaddr failed\n", e->e_ctlr);
	   pio_mapfree (pmap);
	   return;
	 }
	xydi[e->e_ctlr].xyd = 0;

	vec = eisa_ivec_alloc (e->e_adap, IRQ_MASK, EISA_EDGE_IRQ);
	if (vec < 0)
	 { cmn_err (CE_WARN, "xyd: unit %d, no irq vector\n", e->e_ctlr);
	   pio_mapfree (pmap);
	   return;
	 }

	eisa_ivec_set (e->e_adap, vec, xydintr, e->e_ctlr);

	/* This is the device initialization sequence described in the
	   XYD manual, page 4-37. The counter initializtion is ignored */

	xyd->cfg1stat = XYD_LRESET1;
	xyd->cfg2 = XYD_LRESET2;
	xyd->cfg3 = 0;
	xyd->cfg1stat = 0;
	xyd->cfg2 = 0;
/*	xyd->cfg4 = 1; */
	xyd->dmaclr1 = 0;
	xyd->dmaclr2 = 0;
	xyd->cntintclr = 0;

	/* Now set the directions */

	xyd->cfg3 = (XYD_WRITEA|XYD_WRITEB);
	xyd->cfg2 = (XYD_DIODEN|XYD_DIOCEN|XYD_PULSE2);

	xydi[e->e_ctlr].xyd = xyd;
	cmn_err (CE_CONT, "xydedtinit: unit %d vector %x\n",
		e->e_ctlr, vec);
}

xydopen(dev,flag,otyp,crp)
dev_t *dev;
int flag;
int otyp;
cred_t *crp;
{
	/* This device is an exclusive open device: only one process
	   may open it at a time. This prevents contention problems
	   and makes the driver software easier to implement. */

	XydSoftc *info;
	volatile Xyd *xyd;
	int oldflags;
	int s;

	info = &xydi[geteminor(*dev)];

	oldflags = info->flags;

	if (! info->xyd)
	 { return (ENXIO);
	 }
	xyd = info->xyd;
	s = splhi();
	if (info->flags & XYD_BUSY)
	 { splx(s);
	   return (EBUSY);
	 }
	else
	 { info->flags |= XYD_BUSY;
	 }
	splx(s);

	info->qsize = TQ_DEFAULT_SIZE;
	info->intrCnt = 0;
	info->pollhead = phalloc (KM_SLEEP);

	xyd->cfg3 = (XYD_WRITEA|XYD_WRITEB);
	xyd->cfg2 = (XYD_DIODEN|XYD_DIOCEN|XYD_PULSE2);

	return (0);
}

static doclose(info)
XydSoftc *info;
{
	volatile Xyd *xyd;
	int s;

	xyd = info->xyd;

	/* This clears interrupts on line 2 */
	xyd->cfg2 = (XYD_DIODEN|XYD_DIOCEN|XYD_PULSE2);

	if (info->pollhead)
	 { phfree (info->pollhead);
	 }
	else
	 { cmn_err (CE_WARN, "xydclose: pollhead is NULL\n");
	 }
	info->flags = 0;
}

xydclose(dev,flag,otyp,crp)
dev_t dev;
int flag;
int otyp;
cred_t *crp;
{
	/* When closing the device, reset the registers to what
	   they were when the device was openned. */

	XydSoftc *info;

	info = &xydi[geteminor(dev)];

	/* check for soft-close */
	if (! (info->flags & XYD_BUSY) )
	 { /* don't return an error, since this may happen normally */
	   return (0);
	 }
	doclose (info);
	return (0);
}

xydread(dev,uiop,crp)
dev_t dev;
uio_t *uiop;
cred_t *crp;
{
	/* The read routine returns a single byte (set to 1) if an interrupt
	   arrived. */

	XydSoftc *info;
	int cnt, s;
	int ack = 1;

	info = &xydi[geteminor(dev)];

	/* check for soft-close */
	if (! (info->flags & XYD_BUSY) )
	 { return (EBADF);
	 }

	s = splhi();
	cnt = info->intrCnt;
	if (cnt)
	 { info->intrCnt--;
	 }
	splx(s);
	if (cnt)
	 { if (ureadc (ack, uiop) != 0)
	    { return (EFAULT);
	    }
	 }
	return (0);
}

xydwrite(dev,uiop,crp)
dev_t dev;
uio_t *uiop;
cred_t *crp;
{
	return (ENXIO);
}

xydchpoll(dev,events,anyyet,reventsp,phpp)
dev_t dev;
short events;
int anyyet;
short *reventsp;
struct pollhead **phpp;
{
	/* Check to see if any interrupt events have arrived. If not,
	   then queue the process for a possible select wakeup. */

	XydSoftc *info;
	int cnt, s;

	info = &xydi[geteminor(dev)];

	/* check for soft-close */
	if (! (info->flags & XYD_BUSY) )
	 { return (EBADF);
	 }

	/* What about races? */
	if ((events & (POLLIN|POLLRDNORM)) && info->intrCnt > 0)
	 { *reventsp = events & (POLLIN|POLLRDNORM);
	 }
	else
	 { *reventsp = 0;
	   if (!anyyet)
	    { *phpp = info->pollhead;
	    }
	 }
	return (0);
}

xydioctl(dev,cmd,arg,mode,crp,rvalp)
dev_t dev;
int cmd;
void *arg;
int mode;
cred_t *crp;
int *rvalp;
{
	XydSoftc *info;

	info = &xydi[geteminor(dev)];

	/* check for soft-close */
	if (! (info->flags & XYD_BUSY) )
	 { return (EBADF);
	 }

	switch (cmd)
	 { default:
	    { return (ENOTTY);
	      break;
	    }
	   case RCIIOC_CONFIRM:
	    { break;
	    }
	   case RCIIOC_INTR_ENABLE:
	    { int mask = (int)arg;
	      volatile Xyd *xyd = info->xyd;
	      int s;

	      s = splhi(); /* is this really necessary? */
	      if (mask)
	       { int temp;
		 /* set interrupts */
		 xyd->cfg2 = (XYD_DIODEN|XYD_DIOCEN|XYD_PULSE2|XYD_INTEN2);
		 temp = xyd->portCD; /* just paranoid; clear interrupts */
	       }
	      else
	       { /* clear interrupts */
		 xyd->cfg2 = (XYD_DIODEN|XYD_DIOCEN|XYD_PULSE2);
	       }
	      splx(s);
	      break;
	    }
	   case RCIIOC_SET_INTRQ:
	    { int size, s;

	      size = (int)arg;
	      if (size <= 0)
	       { return (EINVAL);
	       }
	      else
	       { s = splhi();
		 info->qsize = size;
		 info->intrCnt = 0;
		 splx(s);
	       }
	      break;
	    }
	   case RCIIOC_GET_INTRQ:
	    { int size;

	      size = info->qsize;
	      if (copyout ((caddr_t)&size, (caddr_t)arg, sizeof(int)))
	       { return (EFAULT);
		 break;
	       }
	      break;
	    }
/* RCIIOC_SOFT_CLOSE is a special hack just for the SGI machines, since
   it seems there is a bug: calling close(2) does not always result
   in the driver close routine being called right away. So we fake
   it with an ioctl, just to be sure. What a botch! */
	   case RCIIOC_SOFT_CLOSE:
	    { doclose (info);      
	      break;
	    }
	 }
	return (0);
}
