#ifndef _S16DREG_H_
#define _S16DREG_H_

#include "rcisys.h"
#include <sys/types.h>
#include <sys/poll.h>

typedef struct {              /* registers 				*/
	u_short  csr ;	 	/* command/status register 		*/
	u_short  config ;	/* configuration register               */
	u_short  dummy1 ;
	u_short  count ;	/* word count 				*/
				/* set count to 1 less than desired	*/
	u_int	 dmaaddr ;	/* DMA address low			*/
	u_int    lastaddr ;	/* last address low			*/
				/* address of last word to xfer 	*/
	u_short  data;		/* Data register			*/
	u_short  dummy2;	/* for alignment			*/
} S16dRegs;

/* command register bits */

#define S16D_GO         0x0001  /* start DMA */
#define S16D_DIR        0x0002  /* select DMA direction */
#define S16D_FLUSH      0x0004  /* aborts DMA */
#define S16D_FNCT1      0x0010  /* application defined output bits */
#define S16D_FNCT2	0x0020
#define S16D_FNCT3	0x0040	

#define S16D_INIT       0x0080  /* reset device */
#define S16D_BCLR       0x0100  /* abort DMA and reset fifos */
#define S16D_FCLK       0x1000  /* fake CLK input cycle */
#define S16D_RDINT      0x2000  /* clear interrupt */
#define S16D_REOD       0x4000  /* clear end-of-DMA interrupt */
#define S16D_DISINT     0x8000  /* disables interrupts when set */

#define S16D_RESET (S16D_BCLR | S16D_RDINT | S16D_REOD | S16D_DISINT)
/* command register bits that need to be saved in software */
#define S16D_SAVE_MASK \
    (S16D_DIR|S16D_INIT|S16D_FNCT1|S16D_FNCT2|S16D_FNCT3|S16D_DISINT)

/* status register bits */

#define S16D_DMAON_S    0x0001  /* dma in progress */
#define S16D_DIR_S      0x0002  /* complement of S16D_DIR in command reg */
#define S16D_DINTDIS_S  0x0004  /* complement of S16D_DINTDIS in config reg */
#define S16D_EODDIS_S   0x0008  /* complement of S16D_EODDIS in config reg */

#define S16D_FNCT1_S    0x0010  /* values of S16D_FNCTX in command reg */
#define S16D_FNCT2_S    0x0020
#define S16D_FNCT3_S    0x0040
#define S16D_INIT_S   	0x0080  /* complement of S16D_INIT in command reg */

#define S16D_STATA_S    0x0100  /* application defined external inputs */
#define S16D_STATB_S    0x0200
#define S16D_STATC_S    0x0400

#define S16D_DINT_S     0x0800  /* state of external line DINT */
#define S16D_INTDEV_S   0x2000  /* interrupt pending; cleared by S16D_RDINT */
#define S16D_EODMA_S    0x4000  /* End of DMA interrupt pending; cleared by
				   S16D_REOD */   
#define S16D_INT_S      0x8000  /* interrupts are pending or enabled */

/* command bits which can be read back directly by the status register */
#define S16D_CMD_STORED		(S16D_FNCT1_S | S16D_FNCT2_S | S16D_FNCT3_S)
/* command bits whose complement can be read back by the status register */
#define S16D_CMD_STORED_I	(S16D_INIT_S | S16D_DIR_S)

/* these command bits are left "as-is" when the device is closed or opened */
#define S16D_FREE_CMD_BITS	(S16D_FNCT1 | S16D_FNCT2 | S16D_FNCT3 | S16D_DIR)

/* configuration register bits */

#define S16D_DACKP      0x0001  /* sets polarity of DACK signal */
#define S16D_CLKP       0x0002  /* data clock polarity */
#define S16D_DBMODE	0x0004	/* disable SBus burst DMA transfers */
#define S16D_DFRST	0x0008	/* disable fifo reset, so that setting BCLR */
				/* will not reset the fifos */
#define S16D_SWAP       0x0100  /* swap bytes on input/output words */
#define S16D_ENDINT	0x2000  /* device interrupt enable. Need DISINT */
				/* cleared as well. */
#define S16D_ENEOD	0x4000  /* end-of-dma interrupt enable. Need DISINT */
				/* cleared as well. */

#ifdef _KERNEL

typedef struct {
	dev_info_t *dip;
	int instance;
	int hasHighLevelIntr;
	/*volatile*/ struct pollhead pollhead;

	ddi_iblock_cookie_t iblkCookie;
	ddi_idevice_cookie_t idevCookie;
	kmutex_t intrMux;

	/* fields for low-level interrupt handler, if needed */
	ddi_iblock_cookie_t softIblkCookie;
	kmutex_t lowMux;
	int softIntrRunning;
	ddi_softintr_t softIntrId;
	int stray;

	kmutex_t *muxp;

#define S16D_INTR_MUTEX_SET	0x01
#define S16D_LOW_MUTEX_SET	0x02
#define S16D_IHANDLER_SET	0x04
#define S16D_SOFT_IHANDLER_SET	0x08
#define S16D_REGS_MAPPED	0x10
#define S16D_MAIN_MINOR_CREATED 0x20
#define S16D_MMAP_MINOR_CREATED 0x40
	int resources;

	volatile int highIntrCnt;
	volatile int intrCnt;
	volatile int qsize;
#define S16D_BUSY	0x01
	volatile int flags;

	S16dRegs *regs;
	int intrEnable; /* needed because DISINT is not stored */
	u_short config;	/* needed because config is write-only (GRRR!) */
} S16dSoftc;

#endif

#endif
